package com.hzya.frame.sys.integrationtask.classification.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.sys.integrationtask.classification.dao.IIntegrationTaskClassificationDao;
import com.hzya.frame.sys.integrationtask.classification.entity.IntegrationTaskClassificationEntity;
import com.hzya.frame.sys.integrationtask.classification.service.IIntegrationTaskClassificationService;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hzya.frame.basedao.service.impl.BaseService;

import java.util.List;

/**
 * @description: 任务分类  service
 * @tableName: integration_task_classification
 * @entityName: IntegrationTaskClassificationEntity
 * @author: gjh
 * @history: 1.0
 */
@Service(value = "integrationTaskClassificationService")
public class IntegrationTaskClassificationServiceImpl extends BaseService<IntegrationTaskClassificationEntity, String> implements IIntegrationTaskClassificationService {

	protected IIntegrationTaskClassificationDao integrationtaskclassificationDao;

	@Autowired
	public void setIntegrationTaskClassificationDao(IIntegrationTaskClassificationDao dao) {
		this.integrationtaskclassificationDao = dao;
		this.dao = dao;
	}

	/**
	 * 新增任务分类
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity saveTaskClassification(JSONObject jsonObject) {
		IntegrationTaskClassificationEntity integrationTaskClassificationEntity = getData("jsonStr", jsonObject, IntegrationTaskClassificationEntity.class);
		//分类id
		long uuid = UUIDLong.longUUID();
		integrationTaskClassificationEntity.setId(String.valueOf(uuid));
		if(null==integrationTaskClassificationEntity.getClass_name()||"".equals(integrationTaskClassificationEntity.getClass_name())){
			return BaseResult.getFailureMessageEntity("任务分类名称不能为空");
		}
		if(null==integrationTaskClassificationEntity.getClass_code()||"".equals(integrationTaskClassificationEntity.getClass_code())){
			return BaseResult.getFailureMessageEntity("任务分类编码不能为空");
		}
        if(integrationtaskclassificationDao.getCount(integrationTaskClassificationEntity)>0){
			return BaseResult.getFailureMessageEntity("任务分类编码已经存在");
		}
		IntegrationTaskClassificationEntity taskClassificationEntity = integrationtaskclassificationDao.save(integrationTaskClassificationEntity);
		return BaseResult.getSuccessMessageEntity("任务分类保存成功",taskClassificationEntity);
	}

	/**
	 * 修改任务分类
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity updateTaskClassification(JSONObject jsonObject) {
		IntegrationTaskClassificationEntity integrationTaskClassificationEntity = getData("jsonStr", jsonObject, IntegrationTaskClassificationEntity.class);
		if(null==integrationTaskClassificationEntity||"".equals(integrationTaskClassificationEntity)){
			return BaseResult.getFailureMessageEntity("需要修改的菜单树为空");
		}
		if(null==integrationTaskClassificationEntity.getClass_code()||"".equals(integrationTaskClassificationEntity.getClass_code())){
			return BaseResult.getFailureMessageEntity("修改时分类编码不能为空");
		}
		if(null==integrationTaskClassificationEntity.getClass_name()||"".equals(integrationTaskClassificationEntity.getClass_name())){
			return BaseResult.getFailureMessageEntity("修改时分类名称不能为空");
		}
		if(integrationtaskclassificationDao.getCount(integrationTaskClassificationEntity)>0){
			return BaseResult.getFailureMessageEntity("任务分类编码已经存在");
		}
		integrationtaskclassificationDao.update("IntegrationTaskClassificationEntity_update",integrationTaskClassificationEntity);
		return BaseResult.getSuccessMessageEntity("修改成功",integrationTaskClassificationEntity);
	}

	/**
	 * 删除任务分类
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity updateTaskClassificationSts(JSONObject jsonObject) {
		IntegrationTaskClassificationEntity integrationTaskClassificationEntity = getData("jsonStr", jsonObject, IntegrationTaskClassificationEntity.class);
		if(null==integrationTaskClassificationEntity||"".equals(integrationTaskClassificationEntity)){
			return BaseResult.getFailureMessageEntity("需要删除的菜单树为空");
		}
		if(null==integrationTaskClassificationEntity.getId()||"".equals(integrationTaskClassificationEntity.getId())){
			return BaseResult.getFailureMessageEntity("需要删除的菜单树id为空");
		}
		IntegrationTaskClassificationEntity integrationTaskClassification=new IntegrationTaskClassificationEntity();
		integrationTaskClassification.setId(integrationTaskClassificationEntity.getId());
		integrationTaskClassification.setSts("Y");
		//根据id查询数据
		List<IntegrationTaskClassificationEntity> integrationTaskClassificationEntities = integrationtaskclassificationDao.query(integrationTaskClassification);
		//递归删除数据
		recursiveDeleteTaskClassification(integrationTaskClassificationEntities);
		return BaseResult.getSuccessMessageEntity("任务分类删除成功",integrationTaskClassificationEntity.getId());
	}

	/**
	 * 递归删除任务分类
	 * @param Classifications
	 */
	private void recursiveDeleteTaskClassification(List<IntegrationTaskClassificationEntity> Classifications){
       if(null!=Classifications && Classifications.size()>0){
		   for (int i=0;i<Classifications.size();i++){
              IntegrationTaskClassificationEntity integrationTaskClassificationEntity=new IntegrationTaskClassificationEntity();
			  integrationTaskClassificationEntity.setClass_superiors(Classifications.get(i).getId());
			  integrationTaskClassificationEntity.setSts("Y");
			   List<IntegrationTaskClassificationEntity> integrationTaskClassificationEntities = integrationtaskclassificationDao.query(integrationTaskClassificationEntity);
			   if(null !=integrationTaskClassificationEntities && integrationTaskClassificationEntities.size()>0){
				   recursiveDeleteTaskClassification(integrationTaskClassificationEntities);
			   }
             //删除节点
			   IntegrationTaskClassificationEntity integrationTaskClassification=new IntegrationTaskClassificationEntity();
			   integrationTaskClassification.setId(Classifications.get(i).getId());
			   integrationtaskclassificationDao.update("IntegrationTaskClassificationEntity_logicDelete",integrationTaskClassification);
		   }
	   }
	}

	/**
	 * 查询任务分类
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity queryTaskClassification(JSONObject jsonObject) {
		IntegrationTaskClassificationEntity integrationTaskClassificationEntity=new IntegrationTaskClassificationEntity();
		integrationTaskClassificationEntity.setSts("Y");
		integrationTaskClassificationEntity.setLvl("0");
		List<IntegrationTaskClassificationEntity> integrationTaskClassificationEntities = integrationtaskclassificationDao.query(integrationTaskClassificationEntity);
		recursiveQueryTaskClassification(integrationTaskClassificationEntities);
		return BaseResult.getSuccessMessageEntity("查询成功",integrationTaskClassificationEntities);
	}

	/**
	 * 递归查询数据
	 * @param Classifications
	 */
	private void recursiveQueryTaskClassification(List<IntegrationTaskClassificationEntity> Classifications){
		//判断是否有数据
		if(Classifications.size()>0 && Classifications !=null){
			for (int i=0;i<Classifications.size();i++){
				IntegrationTaskClassificationEntity integrationTaskClassificationEntity=new IntegrationTaskClassificationEntity();
				integrationTaskClassificationEntity.setClass_superiors(Classifications.get(i).getId());
				integrationTaskClassificationEntity.setSts("Y");
				List<IntegrationTaskClassificationEntity> integrationTaskClassificationEntities = integrationtaskclassificationDao.query(integrationTaskClassificationEntity);
				//判断查询出来的下级数据是否有数据
				if(integrationTaskClassificationEntities.size()>0 && integrationTaskClassificationEntities !=null){
					recursiveQueryTaskClassification(integrationTaskClassificationEntities);
					Classifications.get(i).setChildClassifications(integrationTaskClassificationEntities);
				}
			}
		}
	}
}
