package com.hzya.frame.sys.integrationtask.living.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.sys.integrationtask.living.dao.IIntegrationTaskLivingDao;
import com.hzya.frame.sys.integrationtask.living.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sys.integrationtask.living.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sys.integrationtask.living.entity.IntegrationTaskLivingEntity;
import com.hzya.frame.sys.integrationtask.living.service.IIntegrationTaskLivingService;
import com.hzya.frame.sys.integrationtask.log.entity.IntegrationTaskLogDetailEntity;
import com.hzya.frame.sys.integrationtask.log.entity.IntegrationTaskLogEntity;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @description: 集成任务实例  service
 * @tableName: integration_task_living
 * @entityName: IntegrationTaskLivingEntity
 * @author: gjh
 * @history: 1.0
 */
@Service(value = "integrationTaskLivingService")
public class IntegrationTaskLivingServiceImpl extends BaseService<IntegrationTaskLivingEntity, String> implements IIntegrationTaskLivingService {

    protected IIntegrationTaskLivingDao integrationtasklivingDao;
    @Autowired
    private IIntegrationTaskLivingDetailsDao integrationTaskLivingDetailsDao;

    @Autowired
    public void setIntegrationTaskLivingDao(IIntegrationTaskLivingDao dao) {
        this.integrationtasklivingDao = dao;
        this.dao = dao;
    }

    /**
     * 实例保存
     *
     * @param jsonObject
     * @return
     */
    @Override

    public JsonResultEntity saveIntegrationTaskLiving(JSONObject jsonObject) {
        IntegrationTaskLivingEntity integrationTaskLivingEntity =jsonObject.toJavaObject(IntegrationTaskLivingEntity.class);
        String msg=null;
        String instructions=null;
        if(null!=jsonObject.getString("instructions")){
            instructions=jsonObject.getString("instructions");
        }
        //判断是否是插件执行  1:是插件   2：不是插件
        if("2".equals(jsonObject.getString("plug"))){
            String info = jsonObject.getString("info");
            //如果info为null，说明返回错误，
            if(null!=info && !"".equals(info)){
                JSONObject jsonObjectInfo = JSONObject.parseObject(info);
                msg = jsonObjectInfo.getString("msg");//最新传输信息
                //获取attribute里面得数据  循环保存明细
                if("false".equals(jsonObjectInfo.getString("success"))){
                    jsonObject.put("instructions",jsonObjectInfo.getString("message"));
                    jsonObject.put("msg",jsonObjectInfo.getString("message"));
                    saveMainAndDetails(integrationTaskLivingEntity,jsonObject);
                }
                if("1".equals(jsonObjectInfo.getString("status"))){
                    jsonObject.put("msg",jsonObjectInfo.getString("headMsg"));
                    saveMainAndDetails(integrationTaskLivingEntity,jsonObject);
                }
            }else{
                jsonObject.put("instructions",instructions);
                jsonObject.put("msg",msg);
                saveMainAndDetails(integrationTaskLivingEntity,jsonObject);
            }
        }
        //如果是插件执行
        else{
            jsonObject.put("instructions",instructions);
            jsonObject.put("msg",msg);
            saveMainAndDetails(integrationTaskLivingEntity,jsonObject);
        }
        return BaseResult.getSuccessMessageEntity("任务实例保存成功", integrationTaskLivingEntity);
    }

    private void saveMainAndDetails(IntegrationTaskLivingEntity integrationTaskLivingEntity,JSONObject jsonObject){
        //实例id
        long uuidLiving  = UUIDLong.longUUID();
        integrationTaskLivingEntity.setId(String.valueOf(uuidLiving));
        integrationTaskLivingEntity.setInstructions(jsonObject.getString("instructions"));
        IntegrationTaskLivingEntity integrationTaskLiving = integrationtasklivingDao.save(integrationTaskLivingEntity);
        //实例明细id
        long uuidDetails = UUIDLong.longUUID();
        IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
        integrationTaskLivingDetailsEntity.setId(String.valueOf(uuidDetails));
        integrationTaskLivingDetailsEntity.setTask_linving_id(String.valueOf(uuidLiving));
        integrationTaskLivingDetailsEntity.setCreate_user_id(jsonObject.getString("create_user_id"));
        integrationTaskLivingDetailsEntity.setModify_user_id(jsonObject.getString("modify_user_id"));
        integrationTaskLivingDetailsEntity.setResult(jsonObject.getString("result"));
        integrationTaskLivingDetailsEntity.setNew_transmit_info(jsonObject.getString("msg"));
        integrationTaskLivingDetailsEntity.setRoot_app_new_data(jsonObject.getString("msg"));
        integrationTaskLivingDetailsEntity.setQuery_condition(jsonObject.getString("query_condition"));
        integrationTaskLivingDetailsEntity.setNew_push_date(new Date());
        integrationTaskLivingDetailsEntity.setRepair_pust("N");
        IntegrationTaskLivingDetailsEntity taskLivingDetailsEntity = integrationTaskLivingDetailsDao.save(integrationTaskLivingDetailsEntity);
    }


    /**
     * 根据task_id查询实例id
     *
     * @param integrationTaskLivingEntity
     * @return
     */
    @Override
    public JsonResultEntity queryIntegrationTaskLiving(IntegrationTaskLivingEntity integrationTaskLivingEntity) {
        if(null==integrationTaskLivingEntity.getId()||"".equals(integrationTaskLivingEntity.getId())){
            return BaseResult.getFailureMessageEntity("请携带任务id");
        }
        List<IntegrationTaskLivingEntity> integrationTaskLivingEntities = integrationtasklivingDao.queryList(integrationTaskLivingEntity, "IntegrationTaskLivingEntity_getIdByTaskId");
        for (IntegrationTaskLivingEntity taskLivingEntity : integrationTaskLivingEntities) {
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity=new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setTask_linving_id(taskLivingEntity.getId());
            integrationTaskLivingDetailsEntity.setModify_user_id(integrationTaskLivingEntity.getModify_user_id());
            //根据任务实例id修改任务实例明细id
            integrationTaskLivingDetailsDao.update("entity_logicDelete_Multi_Condition", integrationTaskLivingDetailsEntity);
        }
        //根据task_id修改任务实例sts
        integrationtasklivingDao.update("entity_logicDelete_Multi_Condition", integrationTaskLivingEntity);
        return null;
    }


    /**
     * 实例补推
     *
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity updateIntegrationTaskLivingRetrofit(JSONObject jsonObject) {
        IntegrationTaskLivingEntity integrationTaskLivingEntity = getData("json", jsonObject, IntegrationTaskLivingEntity.class);
        if (null == integrationTaskLivingEntity.getId() || "".equals(integrationTaskLivingEntity.getId())) {
            return BaseResult.getFailureMessageEntity("补推的时候请携带补推数据的id");
        }
        if (null == integrationTaskLivingEntity.getTask_id() || "".equals(integrationTaskLivingEntity.getTask_id())) {
            return BaseResult.getFailureMessageEntity("补推的时候请携带需要补推任务的id");
        }
        //补推返回结果不成功的
        if ("失败".equals(integrationTaskLivingEntity.getResult())) {
            //调用触发器 触发器中更新实例数据 新增日志数据
            //IntegrationTaskLivingJobUnit.createJob(getJob(jsonObject));
        }
        return null;
    }

    /**
     * 组装需要传递到定时任务中的参数
     *
     * @param jsonObject
     * @return
     */
    private JSONObject getJob(JSONObject jsonObject) {
        JSONObject jsonObjectMap = new JSONObject();
        jsonObjectMap.put("task_name", jsonObject.getString("task_name"));
        jsonObjectMap.put("task_code", jsonObject.getString("task_code"));
        jsonObjectMap.put("sence_id", jsonObject.getString("sence_id"));
        jsonObjectMap.put("task_cron", jsonObject.getString("task_cron"));
        jsonObjectMap.put("task_id", jsonObject.getString("task_id"));
        jsonObjectMap.put("create_user_id", jsonObject.getString("create_user_id"));
        jsonObjectMap.put("modify_user_id", jsonObject.getString("modify_user_id"));
        jsonObjectMap.put("id", jsonObject.getString("id"));
        return jsonObject;
    }

    /**
     * 根据实例id修改实例数据
     *
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity updateIntegrationTaskLiving(JSONObject jsonObject) {
        IntegrationTaskLivingEntity integrationTaskLivingEntity = jsonObject.toJavaObject(IntegrationTaskLivingEntity.class);
        //更新实例数据
        integrationtasklivingDao.update("entity_update", integrationTaskLivingEntity);
        IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
        integrationTaskLivingDetailsEntity.setTask_linving_id(integrationTaskLivingEntity.getId());
        integrationTaskLivingDetailsEntity.setResult(integrationTaskLivingEntity.getResult());
        integrationTaskLivingDetailsEntity.setModify_user_id(integrationTaskLivingEntity.getModify_user_id());
        //更新实例明细数据
        integrationTaskLivingDetailsDao.update("entity_update", integrationTaskLivingDetailsEntity);
        return BaseResult.getSuccessMessageEntity("任务实例修改成功");
    }

    /**
     * 查询单个实例
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity queryTaskLivingOne(JSONObject jsonObject) {
        try {
            IntegrationTaskLivingEntity integrationTaskLivingEntity = getData("jsonStr", jsonObject, IntegrationTaskLivingEntity.class);
            if(null==integrationTaskLivingEntity.getId()||"".equals(integrationTaskLivingEntity.getId())){
                return BaseResult.getFailureMessageEntity("查看单个时请携带id");
            }
            List<IntegrationTaskLivingEntity> integrationTaskLivingEntities = integrationtasklivingDao.queryByLike(integrationTaskLivingEntity);
            if(integrationTaskLivingEntities.size()==0){
                return BaseResult.getFailureMessageEntity("查看单个实例为null");
            }
            IntegrationTaskLivingEntity integrationTaskLiving = integrationTaskLivingEntities.get(0);
            if(null==integrationTaskLiving){
                return BaseResult.getSuccessMessageEntity("查询成功",integrationTaskLiving);
            }
            //查询明细数据
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity=new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setTask_linving_id(integrationTaskLiving.getId());
            List<IntegrationTaskLivingDetailsEntity> integrationTaskLivingDetailsEntities = integrationTaskLivingDetailsDao.query(integrationTaskLivingDetailsEntity);
            JSONObject jsonObjectMap=new JSONObject();
            jsonObjectMap.put("main",integrationTaskLiving);
            List<IntegrationTaskLivingDetailsEntity> list=new ArrayList<>();
            if(integrationTaskLivingDetailsEntities.size()>0){
                for (IntegrationTaskLivingDetailsEntity taskLivingDetailsEntity : integrationTaskLivingDetailsEntities) {
                    list.add(taskLivingDetailsEntity);
                }
            }
            jsonObjectMap.put("details",list);
            return BaseResult.getSuccessMessageEntity("查询单个成功",jsonObjectMap);
        }catch (Exception e){
            logger.info("查询单个失败:"+e.getMessage());
            return BaseResult.getFailureMessageEntity("查询单个失败:"+e.getMessage());
        }
    }

    /**
     * 筛选明细数据为正常与删除
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity queryTaskLivingDetails(JSONObject jsonObject) {
        IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity=new IntegrationTaskLivingDetailsEntity();
        List<IntegrationTaskLivingDetailsEntity> list=null;
        String jsonStr = jsonObject.getString("jsonStr");
        JSONObject jsonObjectStr = JSONObject.parseObject(jsonStr);
        //判断刷选条件是正常还是删除
        String type = jsonObjectStr.getString("type");
        integrationTaskLivingDetailsEntity.setTask_linving_id(jsonObjectStr.getString("task_linving_id"));
        switch (type){
            //正常
            case "1":
                integrationTaskLivingDetailsEntity.setSts("Y");
                list = integrationTaskLivingDetailsDao.queryList(integrationTaskLivingDetailsEntity, "IntegrationTaskLivingDetailsEntity_list_sts");
                break;
            //删除
            case "2":
                integrationTaskLivingDetailsEntity.setSts("N");
                list = integrationTaskLivingDetailsDao.queryList(integrationTaskLivingDetailsEntity, "IntegrationTaskLivingDetailsEntity_list_sts");
                break;
            //全部
            case "3":
                list = integrationTaskLivingDetailsDao.queryList(integrationTaskLivingDetailsEntity, "IntegrationTaskLivingDetailsEntity_list_sts");
                break;
        }
        return BaseResult.getSuccessMessageEntity("查询成功",list);
    }

}
