package com.hzya.frame.sys.integrationtask.log.service.impl;

import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.sys.integrationtask.entity.IntegrationTaskDetailsEntity;
import com.hzya.frame.sys.integrationtask.entity.IntegrationTaskEntity;
import com.hzya.frame.sys.integrationtask.log.dao.IIntegrationTaskLogDetailDao;
import com.hzya.frame.sys.integrationtask.log.entity.IntegrationTaskLogDetailEntity;
import com.hzya.frame.sys.integrationtask.monitoring.entity.IntegrationTaskMonitoringEntity;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.uuid.UUIDUtils;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hzya.frame.sys.integrationtask.log.entity.IntegrationTaskLogEntity;
import com.hzya.frame.sys.integrationtask.log.service.IIntegrationTaskLogService;
import com.hzya.frame.sys.integrationtask.log.dao.IIntegrationTaskLogDao;
import com.hzya.frame.basedao.service.impl.BaseService;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @description: 集成任务日志  service
 * @tableName: integration_task_log
 * @entityName: IntegrationTaskLogEntity
 * @author: gjh
 * @history: 1.0
 */
@Service(value = "integrationTaskLogService")
public class IntegrationTaskLogServiceImpl extends BaseService<IntegrationTaskLogEntity, String> implements IIntegrationTaskLogService {

	protected IIntegrationTaskLogDao integrationtasklogDao;
	@Autowired
	private IIntegrationTaskLogDetailDao integrationTaskLogDetailDao;
	@Resource
	private HttpServletResponse httpServletResponse;

	@Autowired
	public void setIntegrationTaskLogDao(IIntegrationTaskLogDao dao) {
		this.integrationtasklogDao = dao;
		this.dao = dao;
	}

	/**
	 * 查询单个日志
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity queryTaskLogOne(JSONObject jsonObject) {
		try {
			IntegrationTaskLogEntity integrationTaskLogEntity = getData("jsonStr", jsonObject, IntegrationTaskLogEntity.class);
			if(null==integrationTaskLogEntity.getId()||"".equals(integrationTaskLogEntity.getId())){
				return BaseResult.getFailureMessageEntity("查看单个时请携带id");
			}
			List<IntegrationTaskLogEntity> integrationTaskLogEntities = integrationtasklogDao.queryByLike(integrationTaskLogEntity);
			if(integrationTaskLogEntities.size()==0){
				return BaseResult.getFailureMessageEntity("查看单个为null");
			}
			IntegrationTaskLogEntity integrationTaskLog = integrationTaskLogEntities.get(0);
			//查询明细数据
			IntegrationTaskLogDetailEntity integrationTaskLogDetailEntity=new IntegrationTaskLogDetailEntity();
			integrationTaskLogDetailEntity.setTask_linving_id(integrationTaskLog.getId());
			List<IntegrationTaskLogDetailEntity> integrationTaskLogDetailEntityList = integrationTaskLogDetailDao.query(integrationTaskLogDetailEntity);
			JSONObject jsonObjectMap=new JSONObject();
			jsonObjectMap.put("main",integrationTaskLog);
			List<IntegrationTaskLogDetailEntity> list=new ArrayList<>();
			if(integrationTaskLogDetailEntityList.size()>0){
				for (IntegrationTaskLogDetailEntity taskLogDetailEntity : integrationTaskLogDetailEntityList) {
					list.add(taskLogDetailEntity);
				}
			}
			jsonObjectMap.put("details",list);
			return BaseResult.getSuccessMessageEntity("查询单个成功",jsonObjectMap);
		}catch (Exception e){
			logger.info("查询单个失败:"+e.getMessage());
			return BaseResult.getFailureMessageEntity("查询单个失败:"+e.getMessage());
		}
	}

	/**
	 * 刷选明细正常与失败数据还有全部数据
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity queryTaskLogDetail(JSONObject jsonObject) {
		IntegrationTaskLogDetailEntity integrationTaskLogDetailEntity=new IntegrationTaskLogDetailEntity();
		List<IntegrationTaskLogDetailEntity> integrationTaskLogDetailEntityList=null;
		String jsonStr = jsonObject.getString("jsonStr");
		JSONObject jsonObjectStr = JSONObject.parseObject(jsonStr);
		//判断刷选条件是正常还是删除
		String type = jsonObjectStr.getString("type");
		if(null==type||"".equals(type)){
			type="3";
		}
		switch (type){
			//正常
			case "1":
				integrationTaskLogDetailEntity.setResult("执行成功");
				integrationTaskLogDetailEntityList = integrationTaskLogDetailDao.queryByLike(integrationTaskLogDetailEntity);
				break;
			//失败
			case "2":
				integrationTaskLogDetailEntity.setResult("执行失败");
				 integrationTaskLogDetailEntityList = integrationTaskLogDetailDao.queryByLike(integrationTaskLogDetailEntity);
				break;
			//全部
			case "3":
				integrationTaskLogDetailEntityList = integrationTaskLogDetailDao.queryByLike(integrationTaskLogDetailEntity);
				break;
			//查看单个
			case "4":
				integrationTaskLogDetailEntity.setId(jsonObjectStr.getString("id"));
				integrationTaskLogDetailEntityList = integrationTaskLogDetailDao.queryByLike(integrationTaskLogDetailEntity);
				break;
		}
		return BaseResult.getSuccessMessageEntity("查询成功",integrationTaskLogDetailEntityList);
	}


	/**
	 * 保存日志和日志明细
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity saveIntegrationTaskLog(JSONObject jsonObject) {
		IntegrationTaskLogEntity integrationTaskLogEntity = jsonObject.toJavaObject(IntegrationTaskLogEntity.class);
		String msg=null;
		String instructions=null;
		if(null!=jsonObject.getString("instructions")){
			instructions=jsonObject.getString("instructions");
		}
		if("2".equals(jsonObject.getString("plug"))){
			String info = jsonObject.getString("info");
			if(null!=info && !"".equals(info)){
				JSONObject jsonObjectInfo = JSONObject.parseObject(info);
				msg = jsonObjectInfo.getString("msg");//最新传输信息
				//获取attribute里面得数据  循环保存明细
				if("false".equals(jsonObjectInfo.getString("success"))){
					jsonObject.put("instructions",jsonObjectInfo.getString("message"));
					jsonObject.put("msg",jsonObjectInfo.getString("message"));
					saveLog(integrationTaskLogEntity,jsonObject);
				}
				if("1".equals(jsonObjectInfo.getString("status"))){
					jsonObject.put("msg",jsonObjectInfo.getString("headMsg"));
					saveLog(integrationTaskLogEntity,jsonObject);
				}
			}else{
				jsonObject.put("instructions",instructions);
				jsonObject.put("msg",msg);
				saveLog(integrationTaskLogEntity,jsonObject);
			}
		}else{
			jsonObject.put("instructions",instructions);
			jsonObject.put("msg",msg);
			saveLog(integrationTaskLogEntity,jsonObject);
		}
		return BaseResult.getSuccessMessageEntity("任务日志保存成功",integrationTaskLogEntity);
	}

	private void saveLog(IntegrationTaskLogEntity integrationTaskLogEntity,JSONObject jsonObject){
		//日志id
		long uuid = UUIDLong.longUUID();
		integrationTaskLogEntity.setId(String.valueOf(uuid));
		integrationTaskLogEntity.setInstructions(jsonObject.getString("instructions"));
		integrationtasklogDao.save(integrationTaskLogEntity);
		//日志明细id
		long uuidDetails = UUIDLong.longUUID();
		IntegrationTaskLogDetailEntity integrationTaskLogDetailEntity=new IntegrationTaskLogDetailEntity();
		integrationTaskLogDetailEntity.setCreate_user_id(jsonObject.getString("create_user_id"));
		integrationTaskLogDetailEntity.setModify_user_id(jsonObject.getString("modify_user_id"));
		integrationTaskLogDetailEntity.setId(String.valueOf(uuidDetails));
		integrationTaskLogDetailEntity.setTask_linving_id(String.valueOf(uuid));
		integrationTaskLogDetailEntity.setResult(jsonObject.getString("result"));
		integrationTaskLogDetailEntity.setNew_transmit_info(jsonObject.getString("msg"));
		integrationTaskLogDetailEntity.setRoot_app_new_data(jsonObject.getString("msg"));
		integrationTaskLogDetailEntity.setQuery_condition(jsonObject.getString("query_condition"));
		integrationTaskLogDetailEntity.setNew_push_date(new Date());
		integrationTaskLogDetailEntity.setRepair_pust("N");
		integrationTaskLogDetailDao.save(integrationTaskLogDetailEntity);
	}

	/**
	 * 根据任务id task_id查询任务日志id
	 * @param integrationTaskLogEntity
	 * @return
	 */
	@Override
	public JsonResultEntity queryIntegrationTaskLog(IntegrationTaskLogEntity integrationTaskLogEntity) {
		if(null==integrationTaskLogEntity.getTask_id()||"".equals(integrationTaskLogEntity.getId())){
			return  BaseResult.getFailureMessageEntity("请携带任务id");
		}
		List<IntegrationTaskLogEntity> integrationTaskLogEntities = integrationtasklogDao.queryList(integrationTaskLogEntity, "IntegrationTaskLogEntity_getByTaskId");
		for (IntegrationTaskLogEntity taskLogEntity : integrationTaskLogEntities) {
			IntegrationTaskLogDetailEntity integrationTaskLogDetailEntity=new IntegrationTaskLogDetailEntity();
			integrationTaskLogDetailEntity.setTask_linving_id(taskLogEntity.getTask_id());
			integrationTaskLogDetailEntity.setModify_user_id(integrationTaskLogEntity.getModify_user_id());
			//根据任务日志id修改任务日志明细sts
			integrationTaskLogDetailDao.update("entity_logicDelete_Multi_Condition",integrationTaskLogDetailEntity);
		}
		//根基task_id修改任务日志sts
		integrationtasklogDao.update("entity_logicDelete_Multi_Condition",integrationTaskLogEntity);
		return null;
	}


	/**
	 * 日志输出成excel
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity queryTaskLogExcel(JSONObject jsonObject) {
		try {
			IntegrationTaskLogEntity integrationTaskLogEntity = getData("jsonStr", jsonObject, IntegrationTaskLogEntity.class);
			//查询任务与日志的连表查询语句
			List<IntegrationTaskLogEntity> integrationTaskLogEntities = integrationtasklogDao.queryByLike(integrationTaskLogEntity);
			if(integrationTaskLogEntities.size()==0){
				return BaseResult.getFailureMessageEntity("输出的数据为null");
			}
             createExcel(integrationTaskLogEntity,integrationTaskLogEntities);
			return BaseResult.getSuccessMessageEntity("日志输出成功");
		}catch (Exception e){
			logger.info("日志输出失败:"+e.getMessage());
			return BaseResult.getFailureMessageEntity("日志输出失败:"+e.getMessage());
		}
	}

	/**
	 * 创建excel
	 * @param logEntity
	 * @param list
	 * @throws Exception
	 */
	private void createExcel(IntegrationTaskLogEntity logEntity, List<IntegrationTaskLogEntity> list) throws Exception{
		// 通过工具类创建writer，默认创建xls格式
		ExcelWriter excelWriter= ExcelUtil.getWriter(true);
		//设置列名称
		excelWriter.addHeaderAlias("id","日志id");
		excelWriter.addHeaderAlias("task_name","任务名称");
		excelWriter.addHeaderAlias("task_code","任务编码");
		excelWriter.addHeaderAlias("task_tag","任务标识");
		excelWriter.addHeaderAlias("task_classes","类型");
		excelWriter.addHeaderAlias("start_time","开始时间");
		excelWriter.addHeaderAlias("end_time","完成时间");
		excelWriter.addHeaderAlias("diff_time","耗时");
		excelWriter.addHeaderAlias("result","结果");
		excelWriter.addHeaderAlias("server_name","服务器名");
		excelWriter.addHeaderAlias("server_living","服务器实例");
		excelWriter.addHeaderAlias("sts","状态");
		excelWriter.addHeaderAlias("explain","说明");
		//从第1行开始 第0行为列名称
		excelWriter.setCurrentRow(0);
		//导出有别名的表单
		excelWriter.setOnlyAlias(true);
		excelWriter.write(list,true);
		httpServletResponse.setContentType("application/vnd.ms-excel;charset=utf-8");
		//设置excel名称
		String encodeName = URLEncoder.encode(list.get(0).getTask_name(), "UTF-8");
		httpServletResponse.setHeader("Content-Disposition","attachment;filename="+encodeName+".xlsx");
		ServletOutputStream out=httpServletResponse.getOutputStream();
		excelWriter.flush(out, true);
		// 关闭writer，释放内存
		excelWriter.close();
		IoUtil.close(out);
	}
}
