package com.hzya.frame.sys.integrationtask.monitoring.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.sys.integrationtask.dao.IIntegrationTaskDao;
import com.hzya.frame.sys.integrationtask.entity.IntegrationTaskEntity;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.hzya.frame.web.quartz.QuartzJobFactoryUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hzya.frame.sys.integrationtask.monitoring.entity.IntegrationTaskMonitoringEntity;
import com.hzya.frame.sys.integrationtask.monitoring.service.IIntegrationTaskMonitoringService;
import com.hzya.frame.sys.integrationtask.monitoring.dao.IIntegrationTaskMonitoringDao;
import com.hzya.frame.basedao.service.impl.BaseService;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.List;

/**
 * @description: 集成任务任务监控  service
 * @tableName: integration_task_monitoring
 * @entityName: IntegrationTaskMonitoringEntity
 * @author: gjh
 * @history: 1.0
 */
@Service(value = "integrationTaskMonitoringService")
public class IntegrationTaskMonitoringServiceImpl extends BaseService<IntegrationTaskMonitoringEntity, String> implements IIntegrationTaskMonitoringService {

	protected IIntegrationTaskMonitoringDao integrationtaskmonitoringDao;
	@Autowired
	private IIntegrationTaskDao integrationTaskDao;

	@Autowired
	private HttpServletResponse httpServletResponse;

	@Autowired
	public void setIntegrationTaskMonitoringDao(IIntegrationTaskMonitoringDao dao) {
		this.integrationtaskmonitoringDao = dao;
		this.dao = dao;
	}

	/**
	 * 当触发器执行的时候，修改任务监控执行次数
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity updateIntegrationTaskMonitoring(JSONObject jsonObject) {
		String taskId = jsonObject.getString("task_id");
		IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity=new IntegrationTaskMonitoringEntity();
		integrationTaskMonitoringEntity.setTask_id(taskId);
		integrationTaskMonitoringEntity.setPlan_time(jsonObject.getDate("plan_time"));
		integrationTaskMonitoringEntity.setDiff_time(jsonObject.getString("diff_time"));
		//修改最大执行次数
		IntegrationTaskMonitoringEntity integrationTaskMonitoringEntityCountMax = integrationtaskmonitoringDao.queryIntegrationTaskMonitoringEntity_countMax(integrationTaskMonitoringEntity);
		integrationTaskMonitoringEntity.setRun_num(integrationTaskMonitoringEntityCountMax.getRun_num());
		integrationtaskmonitoringDao.updateIntegrationTaskMonitoring(integrationTaskMonitoringEntity);
		return BaseResult.getSuccessMessageEntity("修改任务监控执行次数成功");
	}

	/**
	 * 监控输出 输出任务与监控连表查询的结果
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity queryMonitoringExportation(JSONObject jsonObject) throws Exception{
		try {
			IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = getData("jsonStr", jsonObject, IntegrationTaskMonitoringEntity.class);
			//根据任务查询监控数据
			List<IntegrationTaskMonitoringEntity> integrationTaskMonitoringEntityListBase = integrationtaskmonitoringDao.queryByLike(integrationTaskMonitoringEntity);
			if(integrationTaskMonitoringEntityListBase.size()==0){
				return BaseResult.getFailureMessageEntity("输出的数据为null");
			}
			createExcel(integrationTaskMonitoringEntity,integrationTaskMonitoringEntityListBase);
			return BaseResult.getSuccessMessageEntity("监控输出成功");
		}catch (Exception e){
			logger.info("监控输出失败:"+e.getMessage());
			return BaseResult.getFailureMessageEntity("监控输出失败:"+e.getMessage());
		}
	}

	/**
	 * 监控保存
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity saveIntegrationTaskMonitoring(JSONObject jsonObject) {
		//监控id
		long uuidMonitor = UUIDLong.longUUID();
		IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = jsonObject.toJavaObject(IntegrationTaskMonitoringEntity.class);
		integrationTaskMonitoringEntity.setRun_num(0);
		integrationTaskMonitoringEntity.setId(String.valueOf(uuidMonitor));
		integrationtaskmonitoringDao.save(integrationTaskMonitoringEntity);
		return BaseResult.getSuccessMessageEntity("任务监控保存成功",integrationTaskMonitoringEntity);
	}

	//修改监控sts为N
	@Override
	public JsonResultEntity updateTaskMonitoringSts(JSONObject jsonObject) {
		String taskId = jsonObject.getString("task_id");
		if(null==taskId||"".equals(taskId)){
			return BaseResult.getFailureMessageEntity("删除时请携带任务id");
		}
		IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity=new IntegrationTaskMonitoringEntity();
		integrationTaskMonitoringEntity.setTask_id(taskId);
		integrationTaskMonitoringEntity.setModify_user_id(jsonObject.getString("modify_user_id"));
		integrationtaskmonitoringDao.update("IntegrationTaskMonitoringEntity_logicDelete_Multi_Condition",integrationTaskMonitoringEntity);
		return BaseResult.getSuccessMessageEntity("监控删除成功");
	}

	//根据taskid判断在监控中是否存在
	@Override
	public List<IntegrationTaskMonitoringEntity> queryTaskMonitoringOne(JSONObject jsonObject) {
		String taskId = jsonObject.getString("task_id");
		List<IntegrationTaskMonitoringEntity> integrationTaskMonitoringEntities=null;
		if(null!=taskId||!"".equals(taskId)){
			IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity=new IntegrationTaskMonitoringEntity();
			integrationTaskMonitoringEntity.setTask_id(taskId);
			 integrationTaskMonitoringEntities = integrationtaskmonitoringDao.query(integrationTaskMonitoringEntity);
		}
		return integrationTaskMonitoringEntities;
	}

	/**
	 * 监控执行与挂起
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity updateTaskMonitoringCarryAndHang(JSONObject jsonObject) {
		IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = getData("jsonStr", jsonObject, IntegrationTaskMonitoringEntity.class);
		//查询单个任务，用于判断任务是开启还是关闭，正常还是不正常
		IntegrationTaskEntity integrationTaskEntity = integrationTaskDao.get(integrationTaskMonitoringEntity.getTask_id());
		if(null==integrationTaskEntity||"".equals(integrationTaskEntity)){
            return BaseResult.getFailureMessageEntity("任务查询为null");
		}
		//判断任务状态是否是开启状态 1:开启  2：关闭
		if("2".equals(integrationTaskEntity.getTask_status())){
          return BaseResult.getFailureMessageEntity("任务状态已经关闭");
		}
		String jsonStr = jsonObject.getString("jsonStr");
		JSONObject jsonObjectStr = JSONObject.parseObject(jsonStr);
		jsonObjectStr.put("jobName",jsonObjectStr.getString("task_name"));
		jsonObjectStr.put("jobCode",jsonObjectStr.getString("task_code"));
		String type = jsonObjectStr.getString("type");
		//判断type是执行还是挂起 1：执行  2：挂起
		switch (type){
			//执行  定时器继续执行
			case "1":
				integrationTaskMonitoringEntity.setType("1");
				integrationtaskmonitoringDao.update(integrationTaskMonitoringEntity);
				QuartzJobFactoryUtils.pauseTrigger(jsonObjectStr);
				QuartzJobFactoryUtils.resumeTrigger(jsonObjectStr);
				break;
			//挂起  定时器关闭
			case "2":
				integrationTaskMonitoringEntity.setType("2");
				integrationtaskmonitoringDao.update(integrationTaskMonitoringEntity);
				QuartzJobFactoryUtils.pauseTrigger(jsonObjectStr);
				break;
		}
		return BaseResult.getSuccessMessageEntity("操作成功");
	}

	/**
	 * 创建excel
	 * @param monitoring
	 * @param list
	 * @throws Exception
	 */
	private void createExcel(IntegrationTaskMonitoringEntity monitoring,List<IntegrationTaskMonitoringEntity> list) throws Exception{
		// 通过工具类创建writer，默认创建xls格式
		ExcelWriter excelWriter= ExcelUtil.getWriter(true);
		//设置列名称
		excelWriter.addHeaderAlias("id","监控id");
		excelWriter.addHeaderAlias("task_name","任务名称");
		excelWriter.addHeaderAlias("task_code","任务编码");
		excelWriter.addHeaderAlias("task_tag","任务标识");
		excelWriter.addHeaderAlias("org_id","部署公司");
		excelWriter.addHeaderAlias("server_name","服务器名称");
		excelWriter.addHeaderAlias("run_num","运行次数");
		excelWriter.addHeaderAlias("sts","状态");
		excelWriter.addHeaderAlias("sorts","序号");
		//从第1行开始 第0行为列名称
		excelWriter.setCurrentRow(0);
		//导出有别名的表单
		excelWriter.setOnlyAlias(true);
		excelWriter.write(list,true);
		httpServletResponse.setContentType("application/vnd.ms-excel;charset=utf-8");
		//设置excel名称
		String encodeName = URLEncoder.encode(list.get(0).getTask_name(), "UTF-8");
		httpServletResponse.setHeader("Content-Disposition","attachment;filename="+encodeName+".xlsx");
		ServletOutputStream out=httpServletResponse.getOutputStream();
		excelWriter.flush(out, true);
		// 关闭writer，释放内存
		excelWriter.close();
		IoUtil.close(out);
	}
}
