package com.hzya.frame.sys.integrationtask.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.google.gson.JsonObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.sys.apireflection.service.impl.ApiHandleServiceImpl;
import com.hzya.frame.sys.integrationtask.dao.IIntegrationTaskDao;
import com.hzya.frame.sys.integrationtask.dao.IIntegrationTaskDetailsDao;
import com.hzya.frame.sys.integrationtask.entity.IntegrationTaskDetailsEntity;
import com.hzya.frame.sys.integrationtask.entity.IntegrationTaskEntity;
import com.hzya.frame.sys.integrationtask.living.service.IIntegrationTaskLivingService;
import com.hzya.frame.sys.integrationtask.log.service.IIntegrationTaskLogService;
import com.hzya.frame.sys.integrationtask.monitoring.entity.IntegrationTaskMonitoringEntity;
import com.hzya.frame.sys.integrationtask.monitoring.service.IIntegrationTaskMonitoringService;
import com.hzya.frame.sys.integrationtask.service.IIntegrationTaskService;
import com.hzya.frame.sys.sysenum.SysEnum;
import com.hzya.frame.util.PluginUtils;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.hzya.frame.web.quartz.QuartzJobFactoryUtils;
import org.quartz.CronExpression;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @description: 集成任务  service
 * @tableName: integration_task
 * @entityName: IntegrationTaskEntity
 * @author: gjh
 * @history: 1.0
 */
@Service(value = "integrationTaskService")
public class IntegrationTaskServiceImpl extends BaseService<IntegrationTaskEntity, String> implements IIntegrationTaskService {
    public static final String ATONCE = SysEnum.ATONCE.getValue();//执行策略  Y：立即执行
    public static final String TIMING = SysEnum.TIMING.getValue();//执行策略  N：定时
    public static final String OPEN = SysEnum.OPEN.getValue();//任务状态  1：开启
    public static final String SHUTDOWN = SysEnum.SHUTDOWN.getValue();//任务状态  2：关闭
    protected IIntegrationTaskDao integrationtaskDao;
    Logger logger = LoggerFactory.getLogger(IntegrationTaskServiceImpl.class);
    @Autowired
    private IIntegrationTaskDetailsDao integrationTaskDetailsDao;//任务明细
    @Autowired
    private IIntegrationTaskMonitoringService integrationTaskMonitoringService;//监控
    @Autowired
    private IIntegrationTaskLivingService integrationTaskLivingService;//实例
    @Autowired
    private IIntegrationTaskLogService integrationTaskLogService;//日志
    @Autowired
    private ApiHandleServiceImpl apiHandleService;

    @Autowired
    public void setIntegrationTaskDao(IIntegrationTaskDao dao) {
        this.integrationtaskDao = dao;
        this.dao = dao;
    }

    /**
     * 查询单个
     *
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity queryIntegrationTaskById(JSONObject jsonObject) {
        IntegrationTaskEntity integrationTaskEntity = getData("jsonStr", jsonObject, IntegrationTaskEntity.class);
        if (null == integrationTaskEntity.getId() || "".equals(integrationTaskEntity.getId())) {
            return BaseResult.getFailureMessageEntity("查看单个时请携带id");
        }
        IntegrationTaskEntity integrationTaskEntityGet = integrationtaskDao.get(integrationTaskEntity.getId());
        if (null == integrationTaskEntityGet) {
            return BaseResult.getSuccessMessageEntity("查询成功", integrationTaskEntityGet);
        }
        //查询明细数据
        IntegrationTaskDetailsEntity integrationTaskDetailsEntity = new IntegrationTaskDetailsEntity();
        integrationTaskDetailsEntity.setParent_id(integrationTaskEntityGet.getId());
        List<IntegrationTaskDetailsEntity> integrationTaskDetailsEntityListBase = integrationTaskDetailsDao.query(integrationTaskDetailsEntity);
        JSONObject jsonObjectMap = new JSONObject();
        jsonObjectMap.put("main", integrationTaskEntityGet);
        List<IntegrationTaskDetailsEntity> list = new ArrayList<>();
        if (integrationTaskDetailsEntityListBase.size() > 0) {
            for (IntegrationTaskDetailsEntity taskDetailsEntity : integrationTaskDetailsEntityListBase) {
                list.add(taskDetailsEntity);
            }
        }
        jsonObjectMap.put("details", list);
        return BaseResult.getSuccessMessageEntity("查询成功", jsonObjectMap);
    }


    /**
     * 集成任务新增
     *
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity saveIntegrationTask(JSONObject jsonObject) throws Exception {
        JSONObject jsonObjectResult = saveOrUpdateDetails(jsonObject);
        JSONObject jsonObjectStr = JSONObject.parseObject(jsonObjectResult.getString("jsonObjectStr"));
        IntegrationTaskEntity integrationTaskEntity = JSONObject.parseObject(jsonObjectResult.getString("integrationTaskEntity")).toJavaObject(IntegrationTaskEntity.class);
        //判断是否插件
        String plug = jsonObjectStr.getString("plug");
        jsonObjectStr.put("plug", plug);
        //1：是插件
        String isPlug = "1";
        //2:不是插件
        String isNotPlug = "2";
        //集成任务执行策略 N定时Y立即执行 当为Y立即执行时候,新增集成任务，实例，监控，日志，当为N时候，新增任务和监控，实例与日志在任务执行之后再新增
        String status = jsonObjectResult.getString("status");
        //开始时间
        Date dateStart = new Date();
        try {
            if (TIMING.equals(status)) {
                //根据cron，定时执行任务
                QuartzJobFactoryUtils.createJob(getJob(jsonObjectStr));
            }
            if (ATONCE.equals(status)) {
                try {
                    JSONObject jsonObjectHand = new JSONObject();
                    if (null == jsonObjectStr.getString("sence_id")) {
                        jsonObjectStr.put("sence_id", jsonObjectStr.getString("task_tag"));
                    }
                    jsonObjectHand.put("ref_id", jsonObjectStr.getString("sence_id"));
                    JSONObject req = new JSONObject();
                    req.put("jsonStr", jsonObjectHand.toJSONString());
                    //定义空的 调用场景方法有返回得话，取返回值
                    jsonObjectStr.put("result", "");
                    jsonObjectStr.put("info", "");
                    jsonObjectStr.put("task_type", "后台任务");
                    //保存监控数据
                    List<IntegrationTaskMonitoringEntity> integrationTaskMonitoringEntities = integrationTaskMonitoringService.queryTaskMonitoringOne(jsonObjectStr);
                    if (integrationTaskMonitoringEntities.size() == 0 || null == integrationTaskMonitoringEntities) {
                        //触发时候，新增监控
                        jsonObjectStr.put("plan_time", new Date());
                        integrationTaskMonitoringService.saveIntegrationTaskMonitoring(jsonObjectStr);
                    }
                    //查询单个,得到上次修改时间，当前时间减去修改时间就是上次耗时时间
                    int diff_timeMon = 0;
                    if (integrationTaskMonitoringEntities.size() == 0) {
                        diff_timeMon = 0;
                    } else {
                        IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = integrationTaskMonitoringEntities.get(0);
                        diff_timeMon = Integer.valueOf((int) (dateStart.getTime() - integrationTaskMonitoringEntity.getModify_time().getTime()));
                    }
                    //将cron表达式转为时间
                    CronExpression cronExpression = new CronExpression(jsonObjectStr.getString("task_cron"));
                    Date nextValidTimeAfter = cronExpression.getNextValidTimeAfter(new Date());
                    String format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(nextValidTimeAfter);
                    jsonObjectStr.put("plan_time", format);
                    jsonObjectStr.put("diff_time", diff_timeMon);
                    //触发时候，修改任务监控执行次数
                    integrationTaskMonitoringService.updateIntegrationTaskMonitoring(jsonObjectStr);
                    if (isPlug.equalsIgnoreCase(plug)) {
                        try {
                            PluginBaseEntity pluginBaseEntity = PluginUtils.getPluginsById(jsonObjectStr.getString("plug_id"));
                            String s = pluginBaseEntity.executeBusiness(req);
                            logger.info("执行插件executeBusiness");
                            if ("执行成功".equals(s)) {
                                jsonObjectStr.put("result", "执行成功");
                            }
                        } catch (Exception e) {
                            jsonObjectStr.put("result", "执行失败");
                            jsonObjectStr.put("instructions", e.getMessage());
                        }
                    } else {
                        JsonResultEntity jsonResultEntity = apiHandleService.executeApi(req);
                        if ("执行成功".equals(jsonResultEntity.getMsg())) {
                            jsonObjectStr.put("result", jsonResultEntity.getMsg());
                            jsonObjectStr.put("info", jsonResultEntity.getAttribute());
                        } else {
                            jsonObjectStr.put("result", "执行失败");
                            jsonObjectStr.put("instructions", jsonResultEntity.getMsg());
                        }
                    }
                } catch (Exception e) {
                    jsonObjectStr.put("result", "执行失败");
                    //保存错误说明
                    jsonObjectStr.put("instructions", e.getMessage());
                    return BaseResult.getFailureMessageEntity("任务保存失败:" + e.getMessage());
                } finally {
                    saveLogAndLiving(jsonObjectStr, dateStart);
                }
            }
            IntegrationTaskEntity integrationTaskSave = integrationtaskDao.save(integrationTaskEntity);
            return BaseResult.getSuccessMessageEntity("任务保存成功", integrationTaskSave);
        } catch (Exception e) {
            return BaseResult.getFailureMessageEntity("任务保存失败:" + e.getMessage());
        }
    }

    /**
     * 拼接组装参数，传到定时器中
     *
     * @param jsonObject
     * @return
     */
    private JSONObject getJob(JSONObject jsonObject) {
        JSONObject jsonObjectMap = new JSONObject();
        jsonObjectMap.put("jobName", jsonObject.getString("task_name"));
        jsonObjectMap.put("jobCode", jsonObject.getString("task_code"));
        jsonObjectMap.put("sceneId", jsonObject.getString("sence_id"));
        jsonObjectMap.put("task_tag", jsonObject.getString("task_tag"));
        jsonObjectMap.put("cronExpression", jsonObject.getString("task_cron"));
        jsonObjectMap.put("task_id", jsonObject.getString("task_id"));
        jsonObjectMap.put("create_user_id", jsonObject.getString("create_user_id"));
        jsonObjectMap.put("modify_user_id", jsonObject.getString("modify_user_id"));
        jsonObjectMap.put("plug_id", jsonObject.getString("plug_id"));
        jsonObjectMap.put("plug", jsonObject.getString("plug"));
        return jsonObjectMap;
    }

    /**
     * 拼接组装参数，传到定时器中
     *
     * @param jsonObject
     * @return
     */
    private JSONObject getJobV2(JSONObject jsonObject) {
        JSONObject jsonObjectMap = new JSONObject();
        jsonObjectMap.put("jobName", jsonObject.getString("task_name"));
        jsonObjectMap.put("jobCode", jsonObject.getString("task_code"));
        jsonObjectMap.put("sceneId", jsonObject.getString("sence_id"));
        jsonObjectMap.put("task_tag", jsonObject.getString("task_tag"));
        jsonObjectMap.put("cronExpression", jsonObject.getString("task_cron"));
//        jsonObjectMap.put("task_id", jsonObject.getString("task_id"));
        //TODO 没有task_id这个字段，通过com.hzya.frame.sys.integrationtask.service.impl.IntegrationTaskServiceImpl.initTask 调用
        jsonObjectMap.put("task_id", jsonObject.getString("id"));
        jsonObjectMap.put("create_user_id", jsonObject.getString("create_user_id"));
        jsonObjectMap.put("modify_user_id", jsonObject.getString("modify_user_id"));
        jsonObjectMap.put("plug_id", jsonObject.getString("plug_id"));
        jsonObjectMap.put("plug", jsonObject.getString("plug"));
        return jsonObjectMap;
    }

    /**
     * 保存或者修改任务明细
     *
     * @param jsonObject
     * @return
     */
    private JSONObject saveOrUpdateDetails(JSONObject jsonObject) {
        IntegrationTaskEntity integrationTaskEntity = JSONObject.parseObject(jsonObject.getString("jsonStr")).toJavaObject(IntegrationTaskEntity.class);
        String jsonStr = jsonObject.getString("jsonStr");
        JSONObject jsonObjectStr = JSONObject.parseObject(jsonStr);
        IntegrationTaskDetailsEntity integrationTaskDetailsEntity = new IntegrationTaskDetailsEntity();
        //如果任务code为空，则是新增，如果不为空，则是修改
        if (null == integrationTaskEntity.getTask_code() || "".equals(integrationTaskEntity.getTask_code())) {
            //查询编码最大值
            IntegrationTaskEntity integrationTaskEntityMaxCode = integrationtaskDao.query(integrationTaskEntity, "IntegrationTaskEntity_maxCode");
            String taskCode = "Task-" + integrationTaskEntityMaxCode.getTask_code();
            integrationTaskEntity.setTask_code(taskCode);
            //任务id
            long uuid = UUIDLong.longUUID();
            integrationTaskEntity.setId(String.valueOf(uuid));
            jsonObjectStr.put("task_id", uuid);//将任务id存起来，定时触发时候需要传到任务实例与任务日志中
            jsonObjectStr.put("server_name", integrationTaskEntity.getServer_name());
            jsonObjectStr.put("task_code", taskCode);
        } else {
            jsonObjectStr.put("task_id", integrationTaskEntity.getId());//将任务id存起来，定时触发时候需要传到任务实例与任务日志中
            //修改任务主表数据
            IntegrationTaskEntity integrationTaskupdate = integrationtaskDao.update(integrationTaskEntity);
            //在修改明细之前先删除明细数据,将明细数据sts修改为N
            integrationTaskDetailsEntity.setParent_id(integrationTaskEntity.getId());
            integrationTaskDetailsEntity.setModify_user_id(jsonObject.getString("modify_user_id"));
            integrationTaskDetailsDao.update("IntegrationTaskDetailsEntity_updateSts", integrationTaskDetailsEntity);
        }
        //重新保存任务明细数据
        //任务明细id
        long uuidDetails = UUIDLong.longUUID();
        integrationTaskDetailsEntity.setId(String.valueOf(uuidDetails));
        integrationTaskDetailsEntity.setParent_id(integrationTaskEntity.getId());
        integrationTaskDetailsEntity.setTask_cron(jsonObjectStr.getString("task_cron"));
        integrationTaskDetailsEntity.setNow_execution(jsonObjectStr.getString("now_execution"));
        integrationTaskDetailsEntity.setCreate_user_id(jsonObjectStr.getString("create_user_id"));
        integrationTaskDetailsEntity.setModify_user_id(jsonObjectStr.getString("modify_user_id"));
        IntegrationTaskDetailsEntity taskDetailsEntity = integrationTaskDetailsDao.save(integrationTaskDetailsEntity);
        JSONObject jsonObjectResult = new JSONObject();
        jsonObjectResult.put("jsonObjectStr", jsonObjectStr);
        jsonObjectResult.put("status", taskDetailsEntity.getNow_execution());
        jsonObjectResult.put("integrationTaskEntity", JSONObject.toJSON(integrationTaskEntity));
        return jsonObjectResult;
    }


    /**
     * 集成任务修改
     *
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity updateIntegrationTask(JSONObject jsonObject) throws Exception {
        JSONObject jsonObjectResult = saveOrUpdateDetails(jsonObject);
        JSONObject jsonObjectStr = JSONObject.parseObject(jsonObjectResult.getString("jsonObjectStr"));
        IntegrationTaskEntity integrationTaskEntity = JSONObject.parseObject(jsonObjectResult.getString("integrationTaskEntity")).toJavaObject(IntegrationTaskEntity.class);
        //判断是否插件
        String plug = jsonObjectStr.getString("plug");
        //1：是插件
        String isPlug = "1";
        //如果任务状态为开启，则需要修改监控的修改时间，修改任务和任务明细数据
        //如果任务为关闭，也需要修改任务和任务明细数据，还需要调整监控sts为N，将监控和日志触发器停止
        //任务状态
        String taskState = integrationTaskEntity.getTask_status();
        //开始时间
        Date dateStart = new Date();
        try {
            if (OPEN.equals(taskState)) {
                //定义启动状态 N定时Y立即执行 当为Y立即执行时候,新增集成任务，实例，监控，日志，当为N时候，新增任务和监控，实例与日志在任务执行之后再新增
                String status = jsonObjectResult.getString("status");
                if (TIMING.equals(status)) {
                    //先关闭监控和日志定时器，然后再根据新的cron表达式开启新的定时器
                    QuartzJobFactoryUtils.pauseJob(getJob(jsonObjectStr));
                    //根据cron，定时执行任务
                    QuartzJobFactoryUtils.createJob(getJob(jsonObjectStr));
                }
                if (ATONCE.equals(status)) {
                    try {
                        jsonObjectStr.put("task_type", "后台任务");
                        JSONObject jsonObjectHand = new JSONObject();
                        if (null == jsonObjectStr.getString("sence_id")) {
                            jsonObjectStr.put("sence_id", jsonObjectStr.getString("task_tag"));
                        }
                        jsonObjectHand.put("ref_id", jsonObjectStr.getString("sence_id"));
                        JSONObject req = new JSONObject();
                        req.put("jsonStr", jsonObjectHand.toJSONString());
                        //立即执行的时候监控+1 查询单个得到上次修改时间，当前时间减去修改时间就是上次耗时时间
                        List<IntegrationTaskMonitoringEntity> integrationTaskMonitoringEntities = integrationTaskMonitoringService.queryTaskMonitoringOne(jsonObjectStr);
                        IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = integrationTaskMonitoringEntities.get(0);
                        int diff_timeMon = Integer.valueOf((int) (dateStart.getTime() - integrationTaskMonitoringEntity.getModify_time().getTime()));
                        jsonObjectStr.put("diff_time", diff_timeMon);
                        //将cron表达式转为时间
                        CronExpression cronExpression = new CronExpression(jsonObjectStr.getString("task_cron"));
                        Date nextValidTimeAfter = cronExpression.getNextValidTimeAfter(new Date());
                        String format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(nextValidTimeAfter);
                        jsonObjectStr.put("plan_time", format);
                        //触发时候，修改任务监控执行次数
                        integrationTaskMonitoringService.updateIntegrationTaskMonitoring(jsonObjectStr);
                        //判断是否是插件执行
                        if (isPlug.equalsIgnoreCase(plug)) {
                            try {
                                PluginBaseEntity pluginBaseEntity = PluginUtils.getPluginsById(jsonObjectStr.getString("plug_id"));
                                String s = pluginBaseEntity.executeBusiness(req);
                                logger.info("执行插件executeBusiness");
                                if ("执行成功".equals(s)) {
                                    jsonObjectStr.put("result", "执行成功");
                                }
                            } catch (Exception e) {
                                jsonObjectStr.put("result", "执行失败");
                                jsonObjectStr.put("instructions", e.getMessage());
                            }
                        } else {
                            JsonResultEntity jsonResultEntity = apiHandleService.executeApi(req);
                            if ("执行成功".equals(jsonResultEntity.getMsg())) {
                                jsonObjectStr.put("result", jsonResultEntity.getMsg());
                                jsonObjectStr.put("info", jsonResultEntity.getAttribute());
                            } else {
                                jsonObjectStr.put("result", "执行失败");
                                jsonObjectStr.put("instructions", jsonResultEntity.getMsg());
                            }
                        }
                    } catch (Exception e) {
                        jsonObjectStr.put("result", "执行失败");
                        jsonObjectStr.put("instructions", e.getMessage());
                        return BaseResult.getFailureMessageEntity("修改失败:" + e.getMessage());
                    } finally {
                        //根据场景返回结果新增日志和实例
                        saveLogAndLiving(jsonObjectStr, dateStart);
                    }
                }
            }
            if (SHUTDOWN.equals(taskState)) {
                //关闭定时器
                QuartzJobFactoryUtils.pauseTrigger(getJob(jsonObjectStr));
            }
            return BaseResult.getSuccessMessageEntity("修改成功", integrationTaskEntity.getId());
        } catch (Exception e) {
            return BaseResult.getFailureMessageEntity("任务保存失败:" + e.getMessage());
        }
    }

    /**
     * 集成任务删除(修改sts状态为N)
     *
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity updateIntegrationTaskSts(JSONObject jsonObject) {
        IntegrationTaskEntity integrationTaskEntity = JSONObject.parseObject(jsonObject.getString("jsonStr")).toJavaObject(IntegrationTaskEntity.class);
        if (null == integrationTaskEntity.getId() || "".equals(integrationTaskEntity.getId())) {
            return BaseResult.getFailureMessageEntity("删除时请携带id");
        }
        //查询单个 得到任务编码与任务名称，用来停止定时器
        IntegrationTaskEntity integrationTaskEntityGet = integrationtaskDao.get(integrationTaskEntity.getId());
        String jsonStr = jsonObject.getString("jsonStr");
        JSONObject jsonObjectStr = JSONObject.parseObject(jsonStr);
        jsonObjectStr.put("task_id", integrationTaskEntity.getId());//将任务id存起来，定时触发时候需要传到任务实例与任务日志中
        jsonObjectStr.put("task_name", integrationTaskEntityGet.getTask_name());
        jsonObjectStr.put("task_code", integrationTaskEntityGet.getTask_code());
        //删除时 需要将任务和任务明细以及监控STS修改为N，将实例和日志定时器取消
        //修改任务主表sts
        integrationtaskDao.update("IntegrationTaskEntity_logicDelete", integrationTaskEntity);
        //修改任务明细STS
        IntegrationTaskDetailsEntity integrationTaskDetailsEntity = new IntegrationTaskDetailsEntity();
        integrationTaskDetailsEntity.setParent_id(integrationTaskEntity.getId());
        integrationTaskDetailsEntity.setModify_user_id(jsonObject.getString("modify_user_id"));
        integrationTaskDetailsDao.update("IntegrationTaskDetailsEntity_updateSts", integrationTaskDetailsEntity);
        //修改任务监控sts
        integrationTaskMonitoringService.updateTaskMonitoringSts(jsonObjectStr);
        //将实例和日志定时器取消掉
        QuartzJobFactoryUtils.pauseTrigger(getJob(jsonObjectStr));
        return BaseResult.getSuccessMessageEntity("删除成功", integrationTaskEntity.getId());
    }

    /**
     * 根据返回的结果新增日志和实例
     */
    @Override
    public void saveLogAndLiving(JSONObject jsonObjectStr, Date dateStart) {
        //结束时间
        Date dateEndLiving = new Date();
        int diff_time = Integer.valueOf((int) (dateEndLiving.getTime() - dateStart.getTime()));
        jsonObjectStr.put("start_time", dateStart);
        jsonObjectStr.put("end_time", dateEndLiving);
        jsonObjectStr.put("diff_time", diff_time);
        integrationTaskLivingService.saveIntegrationTaskLiving(jsonObjectStr);
        //根据返回值保存集成任务日志
        Date dateEndLog = new Date();
        int diff_timeLog = Integer.valueOf((int) (dateEndLog.getTime() - dateStart.getTime()));
        jsonObjectStr.put("end_time", dateEndLog);
        jsonObjectStr.put("diff_time", diff_timeLog);
        integrationTaskLogService.saveIntegrationTaskLog(jsonObjectStr);
    }


    @Override
    public void initTaskByIds(List<String> taskIdList) {
        if (taskIdList.size() == 0){
            return;
        }
        IntegrationTaskEntity integrationTaskEntity = new IntegrationTaskEntity();
        integrationTaskEntity.setTask_status("1");
        integrationTaskEntity.setTaskIdList(taskIdList);
        List<IntegrationTaskEntity> integrationTaskEntityList = integrationtaskDao.query(integrationTaskEntity);
        if (null != integrationTaskEntityList && integrationTaskEntityList.size() > 0) {
            logger.info("正在初始化任务计划>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
            for (IntegrationTaskEntity taskEntity : integrationTaskEntityList) {
                String jsonObjectStr = JSON.toJSONString(taskEntity);
                JSONObject jsonObject = com.alibaba.fastjson.JSON.parseObject(jsonObjectStr);
                QuartzJobFactoryUtils.createJob(getJobV2(jsonObject));
                logger.info("初始化任务：" + taskEntity.getTask_name() + " 任务编号：" + taskEntity.getTask_code() + "完成");
            }
        }
    }

    @Override
    public void thirdInterfaceSaveTest( JSONObject reqJsonObject) {
        JSONObject reqObj = reqJsonObject.getJSONObject("jsonStr");

        JobExecutionContext jobExecutionContext = JSON.parseObject(reqObj.getString("jobExecutionContext"),JobExecutionContext.class);
        JSONObject jobDetails = JSON.parseObject(reqObj.getString("jobDetails"),JSONObject.class);
        String sceneName = reqObj.getString("sceneName");
        String sceneId = reqObj.getString("sceneId");
        String isPlugin = reqObj.getString("isPlugin");
        String cronExpression = reqObj.getString("cronExpression");
        Date dateStart  = reqObj.getDate("dateStart");
        try {
            //TODO 此处需要执行二林哥场景调用方法
            JSONObject jsonObject = new JSONObject();
            if (null == sceneId) {
                sceneId = jobDetails.getString("task_tag");
            }
            jsonObject.put("ref_id", sceneId);
            JSONObject req = new JSONObject();
            req.put("jsonStr", jsonObject.toJSONString());
            jobDetails.put("plan_time", new Date());
            jobDetails.put("task_type", "后台任务");
            List<IntegrationTaskMonitoringEntity> integrationTaskMonitoringEntities = integrationTaskMonitoringService.queryTaskMonitoringOne(jobDetails);
            if (integrationTaskMonitoringEntities.size() == 0 || null == integrationTaskMonitoringEntities) {
                //触发时候，新增监控
                jobDetails.put("plan_time", new Date());
                integrationTaskMonitoringService.saveIntegrationTaskMonitoring(jobDetails);
            }
            //查询单个,得到上次修改时间，当前时间减去修改时间就是上次耗时时间
            int diff_timeMon = 0;
            if (integrationTaskMonitoringEntities.size() == 0) {
                diff_timeMon = 0;
            } else {
                IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = integrationTaskMonitoringEntities.get(0);
                diff_timeMon = Integer.valueOf((int) (dateStart.getTime() - integrationTaskMonitoringEntity.getModify_time().getTime()));
            }
            jobDetails.put("diff_time", diff_timeMon);
            //将cron表达式转为时间
            CronExpression cronExpressionPlan = new CronExpression(cronExpression);
            Date nextValidTimeAfter = cronExpressionPlan.getNextValidTimeAfter(new Date());
            String format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(nextValidTimeAfter);
            jobDetails.put("plan_time", format);
            //触发时候，修改任务监控执行次数
            integrationTaskMonitoringService.updateIntegrationTaskMonitoring(jobDetails);
            // 是否是插件 1是 2否
            String plug = "1";
            if (plug.equalsIgnoreCase(isPlugin)) {
                try {
                    PluginBaseEntity pluginBaseEntity = PluginUtils.getPluginsById(jobDetails.getString("plug_id"));
                    String s = pluginBaseEntity.executeBusiness(req);
                    logger.info("执行插件executeBusiness");
                    if ("执行成功".equals(s)) {
                        jobDetails.put("result", "执行成功");
                    }
                } catch (Exception e) {
                    jobDetails.put("result", "执行失败");
                    jobDetails.put("instructions", e.getMessage());
                }
            } else {
                JsonResultEntity jsonResultEntity = apiHandleService.executeApi(req);
                if ("执行成功".equals(jsonResultEntity.getMsg())) {
                    jobDetails.put("result", jsonResultEntity.getMsg());
                    jobDetails.put("info", jsonResultEntity.getAttribute());
                } else {
                    jobDetails.put("result", "执行失败");
                    jobDetails.put("instructions", jsonResultEntity.getMsg());
                }
            }
            //System.out.println("正在执行定时任务…………");
            logger.info("当前执行的任务表达式：" + cronExpression + "当前执行的任务是：" + jobExecutionContext.getTrigger().getKey() + "场景名称： " + sceneName + "场景ID：" + sceneId);
        } catch (Exception e) {
            logger.info("执行任务失败：" + cronExpression + "当前执行的任务是：" + jobExecutionContext.getTrigger().getKey() + "场景名称： " + sceneName + "场景ID：" + sceneId);
            jobDetails.put("result", "执行失败");
            //保存错误说明
            jobDetails.put("instructions", e.getMessage());
            e.printStackTrace();
        } finally {
            //TODO 写入任务执行日志表
             saveLogAndLiving(jobDetails, dateStart);
        }
    }
}
