package com.hzya.frame.sys.integrationtask.workshop.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.sys.apireflection.dao.IApiReflectionDao;
import com.hzya.frame.sys.apireflection.entity.ApiReflectionEntity;
import com.hzya.frame.sys.apireflection.service.impl.ApiHandleServiceImpl;
import com.hzya.frame.sys.integrationtask.monitoring.entity.IntegrationTaskMonitoringEntity;
import com.hzya.frame.sys.integrationtask.monitoring.service.impl.IntegrationTaskMonitoringServiceImpl;
import com.hzya.frame.sys.integrationtask.service.IIntegrationTaskService;
import com.hzya.frame.sys.integrationtask.workshop.dao.IIntegrationTaskWorkshopDetailsDao;
import com.hzya.frame.sys.integrationtask.workshop.entity.IntegrationTaskWorkshopDetailsEntity;
import com.hzya.frame.sys.integrationtask.workshop.entity.IntegrationTaskWorkshopEntity;
import com.hzya.frame.sys.integrationtask.workshop.dao.IIntegrationTaskWorkshopDao;
import com.hzya.frame.sys.integrationtask.workshop.service.IIntegrationTaskWorkshopService;
import com.hzya.frame.util.PluginUtils;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.hzya.frame.web.exception.BaseSystemException;
import org.quartz.CronExpression;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import javax.annotation.Resource;
import com.hzya.frame.basedao.service.impl.BaseService;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 任务工坊(IntegrationTaskWorkshop)表服务实现类
 *
 * @author xiaoguo
 * @since 2023-08-07 17:03:33
 */
@Service("integrationTaskWorkshopService")
public class IntegrationTaskWorkshopServiceImpl extends BaseService<IntegrationTaskWorkshopEntity, String> implements IIntegrationTaskWorkshopService {
    @Resource
    private ApiHandleServiceImpl apiHandleService;
    @Resource
    private IApiReflectionDao apiReflectionDao;
    @Resource
    private IntegrationTaskMonitoringServiceImpl integrationTaskMonitoringService;
    @Resource
    private IIntegrationTaskService integrationTaskService;

    private IIntegrationTaskWorkshopDao integrationTaskWorkshopDao;

    @Resource
    private IIntegrationTaskWorkshopDetailsDao detailsDao;

    @Autowired
	public void setIntegrationTaskWorkshopDao(IIntegrationTaskWorkshopDao dao) {
		this.integrationTaskWorkshopDao = dao;
		this.dao = dao;
	}

	/****
	 * 重新保存方法，把明细表参数赋值给主表字段
	 * @content:
	 * @author 👻👻👻👻👻👻👻👻 gjh
	 * @date  2023-08-10 10:29
	 * @param
	 * @return com.hzya.frame.web.entity.JsonResultEntity
	 **/
    @Override
    public JsonResultEntity saveJson(JSONObject object) {
        JSONObject mainData =  object.getJSONObject("jsonStr");
        JSONArray workshopDetails =  mainData.getJSONArray("integration_task_workshop_details_Detail");
        IntegrationTaskWorkshopEntity integrationTaskWorkshopEntity = JSON.parseObject(mainData.toJSONString(),IntegrationTaskWorkshopEntity.class);
        integrationTaskWorkshopEntity.setParameter(workshopDetails.toJSONString());
        integrationTaskWorkshopDao.save(integrationTaskWorkshopEntity);
        //保存明细表
        for(Object detailsObj : workshopDetails){
            IntegrationTaskWorkshopDetailsEntity integrationTaskWorkshopDetailsEntity  = JSON.parseObject(detailsObj.toString(),IntegrationTaskWorkshopDetailsEntity.class);
            integrationTaskWorkshopDetailsEntity.setFormmainId(integrationTaskWorkshopEntity.getId());
            integrationTaskWorkshopDetailsEntity.setId( UUIDLong.absLongUUIDStr());
            detailsDao.save(integrationTaskWorkshopDetailsEntity);
        }
        return BaseResult.getSuccessMessageEntity("任务工坊保存成功");
    }

    @Override
    public JsonResultEntity updateJson(JSONObject object) {
        JSONObject mainData =  object.getJSONObject("jsonStr");
        JSONArray workshopDetails =  mainData.getJSONArray("integration_task_workshop_details_Detail");
        IntegrationTaskWorkshopEntity integrationTaskWorkshopEntity = JSON.parseObject(mainData.toJSONString(),IntegrationTaskWorkshopEntity.class);
        integrationTaskWorkshopEntity.setParameter(workshopDetails.toJSONString());
        integrationTaskWorkshopDao.update(integrationTaskWorkshopEntity);
        IntegrationTaskWorkshopDetailsEntity  removeEntity  = new IntegrationTaskWorkshopDetailsEntity();
        removeEntity.setFormmainId(integrationTaskWorkshopEntity.getId());
        detailsDao.logicRemoveMultiCondition(removeEntity);
        //保存明细表
        for(Object detailsObj : workshopDetails){
            IntegrationTaskWorkshopDetailsEntity integrationTaskWorkshopDetailsEntity  = JSON.parseObject(detailsObj.toString(),IntegrationTaskWorkshopDetailsEntity.class);
            integrationTaskWorkshopDetailsEntity.setFormmainId(integrationTaskWorkshopEntity.getId());
            integrationTaskWorkshopDetailsEntity.setId( UUIDLong.absLongUUIDStr());
            detailsDao.save(integrationTaskWorkshopDetailsEntity);
        }
        return BaseResult.getSuccessMessageEntity("任务工坊修改成功");
    }

    /***
	 * 推送接口方法,此处需要根据场景id调用二林哥接口，同时需要处理查询左边数据接口时传递参数
	 * @content:
	 * @author 👻👻👻👻👻👻👻👻 gjh
	 * @date  2023-08-08 9:00
	 * @param
	 * @return void
	 **/
    /*@Override
    public JsonResultEntity pushBusinessInterface(JSONObject requestJsonObj) throws Exception {

        JSONObject jsonData = requestJsonObj.getJSONObject("jsonStr");
        String sceneId = jsonData.getString("sceneid");
        JSONObject parameters = jsonData.getJSONObject("parameter");
        ApiReflectionEntity apiReflectionEntity = apiReflectionDao.get(sceneId);
        if(null == apiReflectionEntity){
            throw  new BaseSystemException("根据场景ID未查询到场景集成！场景ID"+sceneId);
        }
        String plug = apiReflectionEntity.getPlug();
        String plug_id = apiReflectionEntity.getPlug_id();
        //是插件
        String isPlug = "1";
        // 如果是插件，调用插件方法，否则调用二林哥方法
        if(isPlug.equalsIgnoreCase(plug)){
            PluginBaseEntity pluginBaseEntity = PluginUtils.getPluginsById(plug_id);
            if(null != pluginBaseEntity){
                pluginBaseEntity.executeBusiness(jsonData);
            }
        }else{
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("ref_id",sceneId);
            jsonObject.put("left_interface_parameters",parameters.toJSONString());
            JSONObject req = new JSONObject();
            req.put("jsonStr",jsonObject.toJSONString());
            apiHandleService.executeApi(req);
        }
        return BaseResult.getSuccessMessageEntity("推送成功！");

    }*/


    @Override
    public JsonResultEntity pushBusinessInterface(JSONObject requestJsonObj) throws Exception {
        JSONObject jsonData = requestJsonObj.getJSONObject("jsonStr");
        String sceneId = jsonData.getString("sceneid");
        JSONObject parameters = jsonData.getJSONObject("parameter");
        ApiReflectionEntity apiReflectionEntity = apiReflectionDao.get(sceneId);
        if(null == apiReflectionEntity){
            throw  new BaseSystemException("根据场景ID未查询到场景集成！场景ID"+sceneId);
        }
        //定义空的 调用场景方法有返回得话，取返回值
        jsonData.put("result", "");
        jsonData.put("info", "");
        jsonData.put("task_type","前台任务");
        jsonData.put("task_id",jsonData.getString("id"));//将任务工坊id当作任务id
        //保存监控数据
        List<IntegrationTaskMonitoringEntity> integrationTaskMonitoringEntities = integrationTaskMonitoringService.queryTaskMonitoringOne(jsonData);
        if (integrationTaskMonitoringEntities.size() == 0 || null == integrationTaskMonitoringEntities) {
            //触发时候，新增监控
            jsonData.put("plan_time",new Date());
            integrationTaskMonitoringService.saveIntegrationTaskMonitoring(jsonData);
        }
        //查询单个,得到上次修改时间，当前时间减去修改时间就是上次耗时时间
        int diff_timeMon = 0;
        if (integrationTaskMonitoringEntities.size() == 0) {
            diff_timeMon = 0;
        } else {
            IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = integrationTaskMonitoringEntities.get(0);
            diff_timeMon = Integer.valueOf((int) (new Date().getTime() - integrationTaskMonitoringEntity.getModify_time().getTime()));
        }
        jsonData.put("diff_time", diff_timeMon);
        //触发时候，修改任务监控执行次数
        integrationTaskMonitoringService.updateIntegrationTaskMonitoring(jsonData);
        String plug = apiReflectionEntity.getPlug();
        String plug_id = apiReflectionEntity.getPlug_id();
        //是插件
        String isPlug = "1";
        try {
            // 如果是插件，调用插件方法，否则调用二林哥方法
            if(isPlug.equalsIgnoreCase(plug)){
                try {
                    PluginBaseEntity pluginBaseEntity = PluginUtils.getPluginsById(plug_id);
                    if(null != pluginBaseEntity){
                        String s = pluginBaseEntity.executeBusiness(jsonData);
                        if("执行成功".equals(s)){
                            jsonData.put("result","执行成功");
                        }
                    }
                }catch (Exception e){
                    jsonData.put("result","执行失败");
                    jsonData.put("instructions",e.getMessage());
                }
            }else{
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("ref_id",sceneId);
                jsonObject.put("left_interface_parameters",parameters.toJSONString());
                JSONObject req = new JSONObject();
                req.put("jsonStr",jsonObject.toJSONString());
                JsonResultEntity jsonResultEntity = apiHandleService.executeApi(req);
                if ("执行成功".equals(jsonResultEntity.getMsg())) {
                    jsonData.put("result", jsonResultEntity.getMsg());
                    jsonData.put("info", jsonResultEntity.getAttribute());
                }else{
                    jsonData.put("result", "执行失败");
                    jsonData.put("instructions",jsonResultEntity.getMsg());
                }
            }
            return BaseResult.getSuccessMessageEntity("推送成功！");
        }catch (Exception e){
            jsonData.put("result","执行失败");
            //保存错误说明
            jsonData.put("instructions",e.getMessage());
            return BaseResult.getFailureMessageEntity("任务保存失败:"+e.getMessage());
        }finally {
            //保存日志和实例
            integrationTaskService.saveLogAndLiving(jsonData,new Date());
        }
    }
}
