package com.hzya.frame.web.quartz;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.sys.apireflection.service.impl.ApiHandleServiceImpl;
import com.hzya.frame.sys.integrationtask.living.service.impl.IntegrationTaskLivingServiceImpl;
import com.hzya.frame.sys.integrationtask.log.service.impl.IntegrationTaskLogServiceImpl;
import com.hzya.frame.sys.integrationtask.monitoring.dao.IIntegrationTaskMonitoringDao;
import com.hzya.frame.sys.integrationtask.monitoring.entity.IntegrationTaskMonitoringEntity;
import com.hzya.frame.sys.integrationtask.monitoring.service.IIntegrationTaskMonitoringService;
import com.hzya.frame.sys.integrationtask.monitoring.service.impl.IntegrationTaskMonitoringServiceImpl;
import com.hzya.frame.sys.integrationtask.service.impl.IntegrationTaskServiceImpl;
import com.hzya.frame.util.PluginUtils;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 定时任务运行工厂类
 */
@DisallowConcurrentExecution
public class QuartzJobFactory implements Job {
    private static final Logger logger = LoggerFactory.getLogger(QuartzJobFactory.class);
    @Resource
    private ApiHandleServiceImpl apiHandleService;
    @Resource
    private IntegrationTaskLivingServiceImpl integrationTaskLivingService;
    @Resource
    private IIntegrationTaskMonitoringService integrationTaskMonitoringService;
    @Resource
    private IntegrationTaskServiceImpl integrationTaskService;

    @Override
    @DSTransactional()
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        logger.info("执行定时任务");
        /** 获取JOB明细对象*/
        JobDetail jobDetail = jobExecutionContext.getJobDetail();
        /** 获取JobKey*/
        JobKey jobKey = jobDetail.getKey();
        /** 获取JobMap对象，用来获取实体类*/
        Map<String, Object> obDataMap = jobDetail.getJobDataMap();
        //获取job数据
        /*String jobStr = JSON.toJSONString(obDataMap.get(jobKey.getName()));
        logger.info("执行任务：当前任务json" + jobStr);
        JSONObject jobDetails = JSON.parseObject(jobStr);*/
        JSONObject jsonObjectMap = new JSONObject(obDataMap);
        JSONObject jobDetails = JSONObject.parseObject(jsonObjectMap.getString("job"));
        //场景名称
        String sceneName = jobDetails.getString("sceneName");
        //场景ID
        String sceneId = jobDetails.getString("sceneId");
        //原API id
        String sourceApiId = jobDetails.getString("sourceApiId");
        //目标API Id
        String targetApiId = jobDetails.getString("targetApiId");
        //插件ID
        String pluginId = jobDetails.getString("plug_id");
        /*是否是插件*/
        String isPlugin = jobDetails.getString("plug");
        //执行任务的cron表达式
        String cronExpression = jobDetails.getString("cronExpression");
        JSONObject attribute = null;
        //日志和实例开始时间
        Date dateStart = new Date();
        String result = null;
        String info = null;
        try {
            //TODO 此处需要执行二林哥场景调用方法
            JSONObject jsonObject = new JSONObject();
            if (null == sceneId) {
                sceneId = jobDetails.getString("task_tag");
            }
            jsonObject.put("ref_id", sceneId);
            JSONObject req = new JSONObject();
            req.put("jsonStr", jsonObject.toJSONString());
            jobDetails.put("plan_time", new Date());
            jobDetails.put("task_type", "后台任务");
            List<IntegrationTaskMonitoringEntity> integrationTaskMonitoringEntities = integrationTaskMonitoringService.queryTaskMonitoringOne(jobDetails);

            //查询单个,得到上次修改时间，当前时间减去修改时间就是上次耗时时间
            int diff_timeMon = 0;
            if (integrationTaskMonitoringEntities.size() == 0) {
                diff_timeMon = 0;
            } else {
                IntegrationTaskMonitoringEntity integrationTaskMonitoringEntity = integrationTaskMonitoringEntities.get(0);
                diff_timeMon = Integer.valueOf((int) (dateStart.getTime() - integrationTaskMonitoringEntity.getModify_time().getTime()));
            }
            jobDetails.put("diff_time", diff_timeMon);
            //将cron表达式转为时间
            CronExpression cronExpressionPlan = new CronExpression(cronExpression);
            Date nextValidTimeAfter = cronExpressionPlan.getNextValidTimeAfter(new Date());
            String format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(nextValidTimeAfter);
            jobDetails.put("plan_time", format);
            //触发时候，修改任务监控执行次数
//            integrationTaskMonitoringService.updateIntegrationTaskMonitoring(jobDetails);

            // 是否是插件 1是 2否
            String plug = "1";
            if (plug.equalsIgnoreCase(isPlugin)) {
                try {
                    logger.info("根据插件ID获取插件,ID: "+jobDetails.getString("plug_id"));

                    PluginBaseEntity pluginBaseEntity = PluginUtils.getPluginsById(jobDetails.getString("plug_id"));
                     String s = pluginBaseEntity.executeBusiness(req);
                    logger.info("执行插件executeBusiness");
                    if ("执行成功".equals(s)) {
                        jobDetails.put("result", "执行成功");
                    }
                } catch (Exception e) {
                   logger.info("执行插件executeBusiness失败",e);
                    jobDetails.put("result", "执行失败");
                    jobDetails.put("instructions", e.getMessage());
                }
            } else {
                JsonResultEntity jsonResultEntity = apiHandleService.executeApi(req);
                if ("执行成功".equals(jsonResultEntity.getMsg())) {
                    jobDetails.put("result", jsonResultEntity.getMsg());
                    jobDetails.put("info", jsonResultEntity.getAttribute());
                } else {
                    jobDetails.put("result", "执行失败");
                    jobDetails.put("instructions", jsonResultEntity.getMsg());
                }
            }
            if (integrationTaskMonitoringEntities.size() == 0 || null == integrationTaskMonitoringEntities) {
                //触发时候，新增监控
                jobDetails.put("plan_time", new Date());
                integrationTaskMonitoringService.saveIntegrationTaskMonitoring(jobDetails);
            }
            //System.out.println("正在执行定时任务…………");
            logger.info("当前执行的任务表达式：" + cronExpression + "当前执行的任务是：" + jobExecutionContext.getTrigger().getKey() + "场景名称： " + sceneName + "场景ID：" + sceneId);
        } catch (Exception e) {
            logger.info("执行任务失败：" + cronExpression + "当前执行的任务是：" + jobExecutionContext.getTrigger().getKey() + "场景名称： " + sceneName + "场景ID：" + sceneId,e);
            jobDetails.put("result", "执行失败");
            //保存错误说明
            jobDetails.put("instructions", e.getMessage());
            logger.info(e.getMessage());

        } finally {
//            //TODO 写入任务执行日志表
//            integrationTaskService.saveLogAndLiving(jobDetails, dateStart);
////            触发时候，修改任务监控执行次数
//            integrationTaskMonitoringService.updateIntegrationTaskMonitoring(jobDetails);
//            integrationTaskService.saveLogAndLiving(jobDetails, dateStart);
        }
    }
}
