package com.hzya.frame.web.init;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.hzya.frame.datasource.DataSourceUtil;
import com.hzya.frame.datasource.DataSourceUtilProperties;
import com.hzya.frame.sys.dataSource.dao.ISysDataSourceDao;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.ArrayList;
import java.util.List;

/**
 * @Content
 * @Author 👻👻👻👻👻👻👻👻👻👻 gjh
 * @Date 2023-06-20 17:07
 * @Version 1.0
 */
@Component
@Order(998)
public class MyApplicationListener implements ApplicationListener<ApplicationReadyEvent> {
    private Logger logger = LoggerFactory.getLogger(this.getClass());
    @Resource
    private DataSource dataSource;

    @Override
    public void onApplicationEvent(ApplicationReadyEvent applicationReadyEvent) {
        //logger.info("***初始化数据源*******");
        //DynamicRoutingDataSource ds = (DynamicRoutingDataSource) dataSource;
        //DruidDataSource defaultDataSource = createDefaultDataSource(getDefaultDataSourceJsonObject());
        ////创建SqlSessionFactory,添加到MAP缓存
        //DataSourceUtil.createSqlSessionFactoryToCache("master", defaultDataSource);
        //ds.addDataSource("master", defaultDataSource);
        //ds.setPrimary("master");
        //ds.setStrict(true);
        //logger.info("***初始化数据源成功！数据源编码：master ya251 *********");
        ////数据源初始化完成后，执行初始化定时任务
        initTask();
    }


    /****
     * @Content:创建数据源，返回DruidDataSource
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-06-14 9:11
     * @Param [defaultDataSourceProperties]
     * @return com.alibaba.druid.pool.DruidDataSource
     **/
    private DruidDataSource createDefaultDataSource(JSONObject defaultDataSourceProperties) throws RuntimeException {
        //根据datasource_code查询数据源配置信息
        String datasource_code = String.valueOf(defaultDataSourceProperties.get("datasource_code"));
        String source_url = String.valueOf(defaultDataSourceProperties.get("source_url"));
        String login_name = String.valueOf(defaultDataSourceProperties.get("login_name"));
        String password = String.valueOf(defaultDataSourceProperties.get("password"));
        String db_name = String.valueOf(defaultDataSourceProperties.get("db_name"));
        String source_type = String.valueOf(defaultDataSourceProperties.get("source_type"));
        if (defaultDataSourceProperties != null) {
            String driveClass = "";
            if ("mysql".equalsIgnoreCase(source_type)) {
                driveClass = DataSourceUtilProperties.MYSQLDRIVER_6;
            } else if ("oracle".equalsIgnoreCase(source_type)) {
                driveClass = DataSourceUtilProperties.ORACLEDRIVER;
            } else if ("sqlserver2000".equalsIgnoreCase(source_type)) {
                driveClass = DataSourceUtilProperties.SQL2000DRIVER;
            } else if ("sqlServer".equalsIgnoreCase(source_type)) {
                driveClass = DataSourceUtilProperties.SQL2005DRIVER;
            } else if ("dm".equalsIgnoreCase(source_type)) {
                driveClass = DataSourceUtilProperties.dm;
            }
            try {
                //测试连接
                Class.forName(driveClass);
                Connection connection = DriverManager.getConnection(source_url, login_name, password);// 相当于连接数据库
                if (connection != null) {
                    //初始化数据源信息
                    DruidDataSource druidDataSource = new DruidDataSource();
                    druidDataSource.setUrl(source_url);
                    druidDataSource.setUsername(login_name);
                    druidDataSource.setPassword(password);
                    druidDataSource.setName(db_name);
                    druidDataSource.setDriverClassName(driveClass);
                    // 连接池连接信息
                    druidDataSource.setInitialSize(40);//初始化时建立物理连接的个数。初始化发生在显示调用init方法，或者第一次getConnection时
                    druidDataSource.setMaxActive(1000); //最大连接池数量 0表示无限制
                    druidDataSource.setMaxWait(28000); //获取连接时最大等待时间，单位毫秒。当链接数已经达到了最大链接数的时候，应用如果还要获取链接就会出现等待的现象，等待链接释放并回到链接池，如果等待的时间过长就应该踢掉这个等待，不然应用很可能出现雪崩现象
                    druidDataSource.setMinIdle(20); //最小空闲连接池数量
                    //datasource.setMaxIdle(30); //最大空闲连接池数量 0表示无限制
                    druidDataSource.setPoolPreparedStatements(false); //是否缓存preparedStatement，也就是PSCache。PSCache对支持游标的数据库性能提升巨大，比如说oracle。在mysql下建议关闭。
                    druidDataSource.setMaxPoolPreparedStatementPerConnectionSize(20);
                    druidDataSource.setTestOnBorrow(true); //申请连接时执行validationQuery检测连接是否有效，这里建议配置为TRUE，防止取到的连接不可用
                    druidDataSource.setTestWhileIdle(true);//建议配置为true，不影响性能，并且保证安全性。申请连接的时候检测，如果空闲时间大于timeBetweenEvictionRunsMillis，执行validationQuery检测连接是否有效。
                    if ("oracle".equals(source_type)) {
                        druidDataSource.setValidationQuery("select 1 from dual");//用来检测连接是否有效的sql，要求是一个查询语句。如果validationQuery为null，testOnBorrow、testOnReturn、testWhileIdle都不会起作用。
                    } else {
                        druidDataSource.setValidationQuery("SELECT 1");
                    }
                    //datasource.setFilters("stat,wall");//属性类型是字符串，通过别名的方式配置扩展插件，常用的插件有：监控统计用的filter:stat日志用的filter:log4j防御sql注入的filter:wall
                    druidDataSource.setTimeBetweenEvictionRunsMillis(60000); //配置间隔多久才进行一次检测，检测需要关闭的空闲连接，单位是毫秒
                    druidDataSource.setMinEvictableIdleTimeMillis(300000); //配置一个连接在池中最小生存的时间，单位是毫秒，这里配置为3分钟180000
                    druidDataSource.setKeepAlive(true); //打开druid.keepAlive之后，当连接池空闲时，池中的minIdle数量以内的连接，空闲时间超过minEvictableIdleTimeMillis，则会执行keepAlive操作，即执行druid.validationQuery指定的查询SQL，一般为select * from dual，只要minEvictableIdleTimeMillis设置的小于防火墙切断连接时间，就可以保证当连接空闲时自动做保活检测，不会被防火墙切断
                    druidDataSource.setRemoveAbandoned(true); //是否移除泄露的连接/超过时间限制是否回收。
                    druidDataSource.setRemoveAbandonedTimeout(3600); //泄露连接的定义时间(要超过最大事务的处理时间)；单位为秒。这里配置为1小时
                    druidDataSource.setLogAbandoned(true); //移除泄露连接发生是是否记录日志
                    druidDataSource.setTestOnReturn(false);

                    druidDataSource.init();
                    return druidDataSource;
                } else {
                    throw new RuntimeException("测试默认数据库连接失败datasource_code：" + datasource_code);
                }
            } catch (Exception e) {
                logger.info("==================创建数据源失败===================");
                throw new RuntimeException("创建数据源失败" + e.getMessage());
            }

        } else {
            throw new RuntimeException("source_code:" + datasource_code + "不存在");
        }
    }


    /****
     * @Content:创建默认数据源配置文件
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-06-14 9:09
     * @Param []
     * @return com.alibaba.fastjson.JSONObject
     **/
    private JSONObject getDefaultDataSourceJsonObject() {
        JSONObject defaultDataSourceProperties = new JSONObject();

        ///////////////////////////////////////////////////////////公司服务器
        defaultDataSourceProperties.put("datasource_code", "ya251");
        defaultDataSourceProperties.put("source_url", "jdbc:mysql://hzya.ufyct.com:9096/businesscenternew?serverTimezone=UTC&useUnicode=true&characterEncoding=UTF8&serverTimezone=GMT%2B8&zeroDateTimeBehavior=convertToNull&useSSL=false&allowLoadLocalInfile=false&autoReconnect=true&failOverReadOnly=false&connectTimeout=30000&socketTimeout=30000&autoReconnectForPools=true");
        defaultDataSourceProperties.put("source_type", "mysql");
        defaultDataSourceProperties.put("login_name", "root");
        defaultDataSourceProperties.put("password", "hzya1314");
        defaultDataSourceProperties.put("db_name", "businesscenternew");


        //defaultDataSourceProperties.put("datasource_code", "bopro");
        //defaultDataSourceProperties.put("source_url", "jdbc:mysql://127.0.0.1:3306/businesscenter?serverTimezone=UTC&useUnicode=true&characterEncoding=UTF8&serverTimezone=GMT%2B8&zeroDateTimeBehavior=convertToNull&useSSL=false&allowLoadLocalInfile=false&autoReconnect=true&failOverReadOnly=false&connectTimeout=30000&socketTimeout=30000&autoReconnectForPools=true");
        //defaultDataSourceProperties.put("source_type", "mysql");
        //defaultDataSourceProperties.put("login_name", "root");
        //defaultDataSourceProperties.put("password", "Bh1591+-j");
        //defaultDataSourceProperties.put("db_name", "businesscenter");

        /////////////////////////////////////////////////////////// 公司达梦数据库
        //defaultDataSourceProperties.put("datasource_code", "businesscenter");
        //defaultDataSourceProperties.put("source_url", "jdbc:dm://hzya.ufyct.com:9040?schema=businesscenter&characterEncoding=UTF-8");
        //defaultDataSourceProperties.put("source_type", "dm");
        //defaultDataSourceProperties.put("login_name", "HZYAZT");
        //defaultDataSourceProperties.put("password", "hzya@1314");
        //defaultDataSourceProperties.put("db_name", "businesscenter");

        ///////////////////////////////////////////////////////////本地服务器
//        defaultDataSourceProperties.put("datasource_code", "ya251");
//        defaultDataSourceProperties.put("source_url", "jdbc:mysql://localhost:3306/businesscenter?serverTimezone=UTC&useUnicode=true&characterEncoding=UTF8&serverTimezone=GMT%2B8&zeroDateTimeBehavior=convertToNull&useSSL=false");
//        defaultDataSourceProperties.put("source_type", "mysql");
//        defaultDataSourceProperties.put("login_name", "root");
//        defaultDataSourceProperties.put("password", "seeyon123456");
//        defaultDataSourceProperties.put("db_name", "businesscenter");

        ///////////////////////////////////////////////////////////速网服务器
//        defaultDataSourceProperties.put("datasource_code", "ya251");
//        defaultDataSourceProperties.put("source_url", "jdbc:mysql://124.71.200.210:10216/businesscenter?serverTimezone=UTC&useUnicode=true&characterEncoding=UTF8&serverTimezone=GMT%2B8&zeroDateTimeBehavior=convertToNull&useSSL=false");
//        defaultDataSourceProperties.put("source_type", "mysql");
//        defaultDataSourceProperties.put("login_name", "root");
//        defaultDataSourceProperties.put("password", "jSvtCVdRZR@10216");
//        defaultDataSourceProperties.put("db_name", "businesscenter");
        return defaultDataSourceProperties;
    }

    private void initTask() {
        //需要初始化的taskID集合，如果默认不增加则查询所有的任务计划
        List<String> taskIdList = new ArrayList<>();
        //如不需要系统重新初始化任务计划，放开
//        taskIdList.add("系统不初始化插件");
        //杭泰致远销售合同推送钉钉任务
//        taskIdList.add("3497196579928427970");
        //杭泰BIP供应商传递OA
//        taskIdList.add("1343295077748918281");
        //杭泰BIP组织传递OA
//        taskIdList.add("1410297000456637817");
        //=====================速网start===================
//        //速网U8C存货档案生成旺店通货品档案
//        taskIdList.add("5480192946193190032");
//        //速网U8C供应商档案生成旺店通供应商档案
//        taskIdList.add("-3655259174701414288");
//        //速网U8C采购订单推送旺店通采购订单
//        taskIdList.add("8470345846677980957");
//        //速网旺店通采购入库单，生成U8C采购入库
//        taskIdList.add("-3281025020354081194");
//        //速网U8C红字采购订单，推送旺店通采购退货单
//        taskIdList.add("-8474333912293880929");
//        //速网旺店通采购退货出库单，生成U8C红字采购入库
//        taskIdList.add("-1925311264968064672");
//        //OMS达人寄样销售出库生成U8C销售订单
//        taskIdList.add("-2069765657546371358");
//        //达人寄样退货入库生成U8C红字销售销售出库单
//        taskIdList.add("-2601013958036799088");
//        //速网(TOC)旺店通换货出库，生成U8C销售订单
//        taskIdList.add("6073023852888933683");
//        //速网(TOB)红字销售订单同步旺店通退换单
//        taskIdList.add("-6740993577531585903");
//        //速网(TOB)旺店通退货入库单生成U8C红字销售出库单
//        taskIdList.add("8186943298008796570");
//        //速网(TOC)退货入库单生成U8C红字销售订单
//        taskIdList.add("-7347438162969149145");
//        //速网(TOB)旺店通销售出库单，生成U8C销售出库单
//        taskIdList.add("2307940401916365256");
//        //速网(TOC)旺店通销售出库单生成U8C销售订单
//        taskIdList.add("4313657280852938148");
//        //速网(TOB)U8C销售订单，推送旺店通原始订单
//        taskIdList.add("-1508341403276391185");
//
//        //240142旺店通生产出库单推送U8C其他出库单202401041457
//        taskIdList.add("-1727593887180373437");
//        //240104速网U8C生产组装拆分单，同步旺店通生产单推送202401041501
//        taskIdList.add("-4332012689203573504");
//        //240104旺店通生产入库单推送U8C其他入库单202401041517
//        taskIdList.add("-9101527738616233282");
//        //240104旺店通盘亏出库、(报废)其他出库,同步u8c其他出库单202401041559
//        taskIdList.add("2565596441674991058");
//        //240104旺店通盘盈入库、其他入库,同步u8c其他入库单202401041607
//        taskIdList.add("337296389084038464");
//        //240104速网U8C调拨订单，推送旺店通调拨单202401041539
//        taskIdList.add("6800106772736788841");
//        //240117旺店通调拨出库单推送速网U8C调拨出库单202401171718
//        taskIdList.add("6268657196004560868");
//        //240117旺店通调拨入库单推送速网U8C调拨入库单202401171719
//        taskIdList.add("7472681518255458048");


        //=====================速网end=====================

        // =====================测试===================
        //taskIdList.add("406774952091143809");
        //taskIdList.add("7445578911194825754");
        //taskIdList.add("3313595368368981850");
        //taskIdList.add("3497196579928427970");
        //taskIdList.add("3497196579928427970");
        //taskIdList.add("-6468742747231859632");
        //taskIdList.add("-8410742166114775252");
        //=====================测试=====================
        //初始化定时任务
        //if (null != iIntegrationTaskService) {
        //    iIntegrationTaskService.initTaskByIds(taskIdList);
        //}
    }

}
